@@ -1,6 +1,6 @@
 ;;;; batch-driver.scm - Driver procedure for the compiler
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -33,7 +33,9 @@
 ; Germany
 
 
-(declare (unit driver))
+(declare
+  (unit driver)
+  (disable-warning var))
 
 
 #{compiler
@@ -41,18 +43,17 @@
   default-standard-bindings default-extended-bindings side-effecting-standard-bindings
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
-  compiler-cleanup-hook check-global-exports
-  foreign-type-table-size file-io-only parse-easy-ffi
+  compiler-cleanup-hook check-global-exports disabled-warnings check-global-imports
+  file-io-only
   unit-name insert-timer-checks used-units inline-max-size
-  debugging perform-lambda-lifting! disable-stack-overflow-checking no-c-syntax-checks
-  register-ffi-macro ffi-include-path-list ffi-dont-include
-  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
-  target-heap-size target-stack-size try-harder target-heap-growth target-heap-shrinkage
+  debugging perform-lambda-lifting! disable-stack-overflow-checking
+  foreign-declarations emit-trace-info block-compilation line-number-database-size
+  target-heap-size target-stack-size target-heap-growth target-heap-shrinkage
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
-  target-initial-heap-size split-level postponed-initforms
+  target-initial-heap-size postponed-initforms
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  broken-constant-nodes inline-substitutions-enabled enable-sharp-greater-read-syntax
+  broken-constant-nodes inline-substitutions-enabled
   emit-profile profile-lambda-list profile-lambda-index profile-info-vector-name
   direct-call-ids foreign-type-table first-analysis emit-closure-info emit-line-info
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
@@ -60,7 +61,7 @@
   reorganize-recursive-bindings substitution-table simplify-named-call emit-unsafe-marker
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! source-filename compressed-literals literal-compression-threshold
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -70,17 +71,19 @@
   topological-sort print-version print-usage initialize-analysis-database dump-exported-globals
   default-declarations units-used-by-default words-per-flonum default-debugging-declarations
   default-profiling-declarations default-optimization-passes compressed-literals-initializer
-  inline-max-size file-requirements
+  inline-max-size file-requirements use-import-table lookup-exports-file
   foreign-string-result-reserve parameter-limit eq-inline-operator optimizable-rest-argument-operators
   membership-test-operators membership-unfold-limit valid-compiler-options valid-compiler-options-with-argument
   chop-separator chop-extension display-real-name-table display-line-number-database explicit-use-flag
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
-  export-list do-lambda-lifting
+  export-list do-lambda-lifting compiler-warning export-file-name
   foreign-argument-conversion foreign-result-conversion}
 
 
 (include "tweaks")
-(include "parameters")
+
+(define-constant default-profile-name "PROFILE")
+(define-constant default-inline-max-size 10)
 
 
 ;;; Compile a complete source file:
@@ -111,16 +114,6 @@
 				   oname) ) ) ]
 		       [(memq 'to-stdout options) #f]
 		       [else (make-pathname #f (if filename (pathname-file filename) "out") "c")] ) ]
-        [partitions (cond [(memq 'split options)
-                           => (lambda (node)
-                                (let ([oname (option-arg node)])
-                                  (cond
-                                    ((symbol? oname)
-                                     (string->number (symbol->string oname)))
-                                    ((string? oname)
-                                     (string->number oname))
-                                    (else oname))))]
-                          [else 1])]
 	[ipath (map chop-separator (string-split (or (getenv "CHICKEN_INCLUDE_PATH") "") ";"))]
 	[opasses default-optimization-passes]
 	[time0 #f]
@@ -139,22 +132,9 @@
 	[a-only (memq 'analyze-only options)]
 	[dynamic (memq 'dynamic options)]
 	[dumpnodes #f]
-	[ffi-mode (memq 'ffi options)]
-	[ffi-parse-mode (memq 'ffi-parse options)]
-	[ofilelist #f]
 	[quiet (memq 'quiet options)]
 	[ssize (or (memq 'nursery options) (memq 'stack-size options))] )
 
-    (define (outfile->list outfile num)
-      (cond
-       [(not outfile) #f]
-       [(= num 1) (list outfile)]
-       [else
-	(let ([file (pathname-file outfile)])
-	  (list-tabulate 
-	   num
-	   (lambda (i) (make-pathname #f (sprintf "~A~A.c" file i))) ) ) ] ) )
-
     (define (cputime) (##sys#fudge 6))
 
     (define (print-header mode dbgmode)
@@ -214,21 +194,8 @@
 	(printf "milliseconds needed for ~a: \t~s~%" pass (- (cputime) time0)) ) )
 
     (define (read-form in)
-      (if (or ffi-mode ffi-parse-mode)
-	  (let ([s (read-string #f in)])
-	    (if (string=? s "")
-		#!eof
-		(cond [ffi-mode
-		       `(declare 
-			  (foreign-declare ,s)
-			  (foreign-parse ,s) ) ]
-		      [else
-		       `(declare
-			  (foreign-parse ,s) ) ] ) ) )
-	  (##sys#read in infohook) ) )
+      (##sys#read in infohook) )
 
-    (when (or (< partitions 1) (not outfile))
-        (set! partitions 1))
     (when uunit
       (set! unit-name (string->c-identifier (stringify (option-arg uunit)))) )
     (set! debugging-chicken 
@@ -237,17 +204,24 @@
 	 (map (lambda (c) (string->symbol (string c)))
 	      (string->list do) ) )
        (collect-options 'debug) ) )
-    (set! try-harder #t)
     (set! dumpnodes (memq '|D| debugging-chicken))
     (when (memq 'lambda-lift options) (set! do-lambda-lifting #t))
     (when (memq 't debugging-chicken) (##sys#start-timer))
     (when (memq 'b debugging-chicken) (set! time-breakdown #t))
+    (and-let* ((xfile (memq 'emit-exports options)))
+      (set! export-file-name (cadr xfile)) )
     (when (memq 'raw options)
       (set! explicit-use-flag #t)
       (set! cleanup-forms '())
       (set! initforms '()) )
     (when (memq 'no-lambda-info options)
       (set! emit-closure-info #f) )
+    (set! use-import-table (memq 'check-imports options))
+    (let ((imps (collect-options 'import)))
+      (when (pair? imps)
+	(set! use-import-table #t)
+	(for-each lookup-exports-file imps) ) )
+    (set! disabled-warnings (map string->symbol (collect-options 'disable-warning)))
     (when (memq 'no-warnings options) 
       (when verbose (printf "Warnings are disabled~%~!"))
       (set! ##sys#warnings-enabled #f) )
@@ -260,7 +234,6 @@
     (when (memq 'disable-interrupts options) (set! insert-timer-checks #f))
     (when (memq 'fixnum-arithmetic options) (set! number-type 'fixnum))
     (when (memq 'block options) (set! block-compilation #t))
-    (when (memq 'disable-c-syntax-checks options) (set! no-c-syntax-checks #t))
     (when (memq 'emit-external-prototypes-first options) (set! external-protos-first #t))
     (when (memq 'inline options) (set! inline-max-size default-inline-max-size))
     (when (memq 'track-scheme options) (set! emit-line-info #t))
@@ -273,12 +246,6 @@
       (when verbose (printf "Identifiers and symbols are case insensitive~%~!"))
       (register-feature! 'case-insensitive)
       (case-sensitive #f) )
-    (and-let* ([slevel (memq 'split-level options)])
-      (set! split-level 
-	(let ([n (string->number (option-arg slevel))])
-	  (if (and n (<= 0 n 2))
-	      n
-	      (quit "invalid argument to `-split-level' option") ) ) ) )
     (when kwstyle
       (let ([val (option-arg kwstyle)])
 	(cond [(string=? "prefix" val) (keyword-style #:prefix)]
@@ -296,18 +263,12 @@
       (append (map chop-separator (collect-options 'include-path))
 	      ##sys#include-pathnames
 	      ipath) )
-    (set! ofilelist (outfile->list outfile partitions))
-    (when (and outfile filename (any (cut string=? <> filename) ofilelist))
+    (when (and outfile filename (string=? outfile filename))
       (quit "source- and output-filename are the same") )
 
     ;; Handle feature options:
     (for-each register-feature! (collect-options 'feature))
 
-    ;; Handle FFI defines and include-paths:
-    (for-each register-ffi-macro (collect-options 'ffi-define))
-    (when (memq 'ffi-no-include options) (set! ffi-dont-include #t))
-    (set! ffi-include-path-list (append (collect-options 'ffi-include-path) ffi-include-path-list))
-
     ;; Load extensions:
     (set! ##sys#features (cons #:compiler-extension ##sys#features))
     (let ([extends (collect-options 'extend)])
@@ -318,7 +279,7 @@
     (set! ##sys#features (delete #:compiler-extension ##sys#features eq?))
 
     (set! ##sys#features (cons '#:compiling ##sys#features))
-    (set! ##sys#features (cons #:match (cons #:match-support ##sys#features)))
+    (set! ##sys#features (cons #:match ##sys#features))
     (##sys#provide 'match) 
 
     ;; Insert postponed initforms:
@@ -342,7 +303,6 @@
 
     (when (memq 'run-time-macros options)
       (set! ##sys#enable-runtime-macros #t) )
-    (enable-sharp-greater-read-syntax)
     (set! target-heap-size
       (if hsize
 	  (arg-val (option-arg hsize))
@@ -387,6 +347,9 @@
 	   (newline) )
 	  ((or (memq 'help options) (memq '-help options) (memq 'h options) (memq '-h options))
 	   (print-usage))
+	  ((memq 'release options)
+	   (display (chicken-version)) 
+	   (newline) )
 	  ((not filename)
 	   (unless quiet
 	     (print-version #t)
@@ -422,9 +385,10 @@
 			     (append (map string->expr prelude)
 				     (reverse forms)
 				     (map string->expr postlude) ) ) )
-			(let* ([f (car files)]
-			       [in (check-and-open-input-file f)] )
-			  (do ([x (read-form in) (read-form in)])
+			(let* ((f (car files))
+			       (in (check-and-open-input-file f)) 
+			       (x1 (read-form in)) )
+			  (do ((x x1 (read-form in)))
 			      ((eof-object? x) 
 			       (close-checked-input-file in f) )
 			    (set! forms (cons x forms)) ) ) ) ] ) ) )
@@ -453,10 +417,9 @@
 				  ',(if unit-name #f profile-name))))
 			     '() )
 			 (map (lambda (pl)
-				`(##core#inline 
-				  "C_i_setslot"
+				`(##sys#set-profile-info-vector!
 				  ,profile-info-vector-name
-				  ',(* profile-info-entry-size (car pl)) 
+				  ',(car pl)
 				  ',(cdr pl) ) )
 			      profile-lambda-list)
 			 (let ([is (fold (lambda (clf r)
@@ -482,11 +445,13 @@
 	       (display-line-number-database) )
 
 	     (when (and block-compilation unit-name)
-	       (warning "compilation of library unit `~a' in block-mode - globals may not be accessible outside this unit"
-			unit-name) )
+	       (compiler-warning 
+		'usage
+		"compilation of library unit `~a' in block-mode - globals may not be accessible outside this unit"
+		unit-name) )
 
 	     (when (and unit-name dynamic)
-	       (warning "library unit `~a' compiled in dynamic mode" unit-name) )
+	       (compiler-warning 'usage "library unit `~a' compiled in dynamic mode" unit-name) )
 
 	     (set! ##sys#line-number-database line-number-database-2)
 	     (set! line-number-database-2 #f)
@@ -538,7 +503,7 @@
 	       (set! ##sys#line-number-database #f)
 	       (set! constant-table #f)
 	       (set! inline-table #f)
-	       (when (and try-harder (not unsafe))
+	       (unless unsafe
 		 (scan-toplevel-assignments (first (node-subexpressions node0))) )
 
 	       (begin-time)
@@ -552,6 +517,7 @@
 		   (begin-time)
 		   (let ([db (analyze-expression node2)])
 		     (when first-analysis
+		       (when use-import-table (check-global-imports db))
 		       (check-global-exports db)
 		       (when (memq 'u debugging-chicken)
 			 (dump-undefined-globals db)) )
@@ -561,7 +527,7 @@
 
 		     (when (memq 's debugging-chicken) (print-program-statistics db))
 
-		     (cond [(and progress (or try-harder (<= i default-optimization-passes)))
+		     (cond [progress
 			    (debugging 'p "optimization pass" i)
 
 			    (begin-time)
@@ -591,8 +557,8 @@
 			    (let ([node3 (perform-closure-conversion node2 db)])
 			      (end-time "closure conversion")
 			      (print-db "final-analysis" '|8| db i)
-			      (when (memq 'e debugging-chicken)
-				(dump-exported-globals db) )
+			      (when export-file-name
+				(dump-exported-globals db export-file-name) )
 			      (let ([upap (user-post-analysis-pass)])
 				(when upap 
 				  (upap db
@@ -601,28 +567,16 @@
 			      (when a-only (exit 0))
 			      (print-node "closure-converted" '|9| node3)
 
-                              (when (and verbose
-					 outfile
-					 (not (> partitions 1)) )
-                                (printf "files to be generated: ~A~%" (string-intersperse ofilelist ", ")) )
-                              
 			      (begin-time)
-			      (receive (node literals lambdas) (prepare-for-code-generation node3 db partitions)
+			      (receive (node literals lambdas) (prepare-for-code-generation node3 db)
 				(end-time "preparation")
 
                                 (begin-time)
-                                (let loopfile [(lof ofilelist)
-                                               (file-partition 0)]
-                                  (let* ([outfile (and lof (car lof))]
-					 [out (if outfile (open-output-file outfile) (current-output-port))] )
-				    (unless quiet
-				      (printf "generating `~A' ...~%" outfile) )
-                                    (generate-code 
-				     literals lambdas out filename dynamic db
-				     (if (= partitions 1) #f file-partition))
-                                    (when outfile (close-output-port out))
-                                    (when (and lof (pair? (cdr lof)))
-                                      (loopfile (cdr lof) (+ file-partition 1)))))
+				(let ((out (if outfile (open-output-file outfile) (current-output-port))) )
+				  (unless quiet
+				    (printf "generating `~A' ...~%" outfile) )
+				  (generate-code literals lambdas out filename dynamic db)
+				  (when outfile (close-output-port out)))
                                 (end-time "code generation")
                                 (when (memq 't debugging-chicken) (##sys#display-times (##sys#stop-timer)))
                                 (compiler-cleanup-hook)
