@@ -1,6 +1,6 @@
 ;;;; c-platform.scm - Platform specific parameters and definitions
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -42,10 +42,10 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home debugging
-  foreign-type-table-size dump-nodes
+  dump-nodes
   unit-name insert-timer-checks used-units inlining
-  foreign-declarations block-compilation analysis-database-size line-number-database-size
-  target-heap-size target-stack-size try-harder default-installation-home 
+  foreign-declarations block-compilation line-number-database-size
+  target-heap-size target-stack-size 
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
@@ -56,7 +56,7 @@
   reorganize-recursive-bindings substitution-table simplify-named-call find-inlining-candidates perform-inlining!
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! decompose-lambda-list rewrite
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -123,17 +123,17 @@
 (define valid-compiler-options
   '(-help h help version verbose explicit-use quiet no-trace no-warnings unsafe block
     check-syntax to-stdout no-usual-integrations case-insensitive no-lambda-info profile inline
-    fixnum-arithmetic disable-interrupts optimize-leaf-routines
+    fixnum-arithmetic disable-interrupts optimize-leaf-routines check-imports
     lambda-lift run-time-macros tag-pointers accumulate-profile
     disable-stack-overflow-checks disable-c-syntax-checks unsafe-libraries raw 
-    emit-external-prototypes-first track-scheme
-    analyze-only dynamic ffi ffi-custom ffi-parse ffi-no-include extension) )
+    emit-external-prototypes-first track-scheme release
+    analyze-only dynamic extension) )
 
 (define valid-compiler-options-with-argument
   '(debug output-file include-path heap-size stack-size unit uses keyword-style require-extension 
-	  inline-limit profile-name
-    prelude postlude prologue epilogue nursery extend feature compress-literals split-level
-    heap-growth heap-shrinkage heap-initial-size ffi-define ffi-include-path split) )
+	  inline-limit profile-name disable-warning emit-exports import
+    prelude postlude prologue epilogue nursery extend feature compress-literals
+    heap-growth heap-shrinkage heap-initial-size ffi-define ffi-include-path) )
 
 
 ;;; Standard and extended bindings:
@@ -161,9 +161,9 @@
 (define default-extended-bindings
   '(bitwise-and bitwise-ior bitwise-xor bitwise-not add1 sub1 fx+ fx- fx* fx/ fxmod
     fx= fx> fx< fx>= fx<= fixnum? fxneg fxmax fxmin identity fp+ fp- fp* fp/ fpmin fpmax fpneg
-    fp> fp< fp= fp>= fp<= atom? fxand fxnot fxior fxxor fxshr fxshl
+    fp> fp< fp= fp>= fp<= fxand fxnot fxior fxxor fxshr fxshl
     arithmetic-shift void flush-output thread-specific thread-specific-set!
-    not-pair? null-list? print print* error cpu-time proper-list? call/cc
+    not-pair? atom? null-list? print print* error cpu-time proper-list? call/cc
     u8vector->byte-vector s8vector->byte-vector u16vector->byte-vector s16vector->byte-vector 
     u32vector->byte-vector 
     s32vector->byte-vector byte-vector-length block-ref block-set! number-of-slots
@@ -176,7 +176,7 @@
     f32vector-ref f64vector-ref
     u8vector-set! s8vector-set! u16vector-set! s16vector-set! u32vector-set! s32vector-set!
     locative-ref locative-set! locative->object locative? global-ref
-    null-pointer? pointer->object) )
+    null-pointer? pointer->object flonum? finite?) )
 
 (define internal-bindings
   '(##sys#slot ##sys#setslot ##sys#block-ref ##sys#block-set!
@@ -185,12 +185,12 @@
     ##sys#check-exact ##sys#check-number ##sys#check-list ##sys#check-pair ##sys#check-string ##sys#check-symbol 
     ##sys#check-char ##sys#check-vector ##sys#check-byte-vector ##sys#list
     ##sys#call-with-values ##sys#fits-in-int? ##sys#fits-in-unsigned-int? ##sys#flonum-in-fixnum-range? 
-    ##sys#fudge ##sys#immediate? ##sys#direct-return
+    ##sys#fudge ##sys#immediate? ##sys#direct-return ##sys#context-switch
     ##sys#make-structure ##sys#apply ##sys#apply-values ##sys#continuation-graft
     ##sys#bytevector? ##sys#make-vector ##sys#setter
     ##sys#foreign-char-argument ##sys#foreign-fixnum-argument ##sys#foreign-flonum-argument
     ##sys#foreign-block-argument ##sys#foreign-number-vector-argument
-    ##sys#foreign-string-argument ##sys#foreign-pointer-argument 
+    ##sys#foreign-string-argument ##sys#foreign-pointer-argument
     ##sys#foreign-integer-argument ##sys#foreign-unsigned-integer-argument ##sys#double->number
     ##sys#peek-fixnum ##sys#setislot ##sys#poke-integer ##sys#permanent? ##sys#values ##sys#poke-double
     ##sys#intern-symbol ##sys#make-symbol ##sys#null-pointer? ##sys#peek-byte) )
@@ -484,6 +484,38 @@
   (rewrite 'values 8 rvalues)
   (rewrite '##sys#values 8 rvalues) )
 
+(let ()
+  (define (rewrite-c-w-v db classargs cont callargs)
+   ;; (call-with-values <var1> <var2>) -> (let ((k (lambda (r) (<var2> <k0> r)))) (<var1> k))
+   ;; - if <var2> is a known lambda of a single argument
+   (and (= 2 (length callargs))
+	(let ((arg1 (car callargs))
+	      (arg2 (cadr callargs)) )
+	  (and (eq? '##core#variable (node-class arg1))	; probably not needed
+	       (eq? '##core#variable (node-class arg2))
+	       (and-let* ((sym (car (node-parameters arg2)))
+			  (val (get db sym 'value)) )
+		 (and (eq? '##core#lambda (node-class val))
+		      (let ((llist (third (node-parameters val))))
+			(and (proper-list? llist)
+			     (= 2 (length (third (node-parameters val))))
+			     (let ((tmp (gensym))
+				   (tmpk (gensym 'r)) )
+			       (debugging 'o "removing single-valued `call-with-values'" (node-parameters val))
+			       (make-node
+				'let (list tmp)
+				(list (make-node
+				       '##core#lambda
+				       (list (gensym 'f_) #f (list tmpk) 0)
+				       (list (make-node
+					      '##core#call '(#t)
+					      (list arg2 cont (varnode tmpk)) ) ) ) 
+				      (make-node
+				       '##core#call '(#t)
+				       (list arg1 (varnode tmp)) ) ) ) ) ) ) ) ) ) ) ) )
+  (rewrite 'call-with-values 8 rewrite-c-w-v)
+  (rewrite '##sys#call-with-values 8 rewrite-c-w-v) )
+
 (rewrite 'values 13 "C_values" #t)
 (rewrite '##sys#values 13 "C_values" #t)
 (rewrite 'call-with-values 13 "C_u_call_with_values" #f)
@@ -541,7 +573,6 @@
 (rewrite 'symbol? 2 1 "C_i_symbolp" #t #f)
 (rewrite 'vector? 2 1 "C_i_vectorp" #t #f)
 (rewrite 'pair? 2 1 "C_i_pairp" #t "C_notvemptyp")
-(rewrite 'atom? 2 1 "C_i_atomp" #t #f)
 (rewrite 'procedure? 2 1 "C_i_closurep" #t #f)
 (rewrite 'port? 2 1 "C_i_portp" #t #f)
 (rewrite 'boolean? 2 1 "C_booleanp" #t #f)
@@ -550,7 +581,9 @@
 (rewrite 'rational? 2 1 "C_i_numberp" #t #f)
 (rewrite 'real? 2 1 "C_i_numberp" #t #f)
 (rewrite 'integer? 2 1 "C_i_integerp" #t #f)
+(rewrite 'flonum? 2 1 "C_i_flonump" #t #f)
 (rewrite 'fixnum? 2 1 "C_fixnump" #t #f)
+(rewrite 'finite? 2 1 "C_i_finitep" #f #f)
 (rewrite '##sys#pointer? 2 1 "C_pointerp" #t #f)
 (rewrite '##sys#generic-structure? 2 1 "C_structurep" #t #f)
 (rewrite 'exact? 2 1 "C_fixnump" #f #f)
@@ -775,6 +808,7 @@
 (rewrite 'vector 16 #f "C_a_i_vector" #t #t)
 (rewrite '##sys#vector 16 #f "C_a_i_vector" #t #t)
 (rewrite '##sys#make-structure 16 #f "C_a_i_record" #t #t)
+(rewrite 'string 16 #f "C_a_i_string" #t #t) ; the last #t is actually too much, but we don't care
 
 (rewrite
  '##sys#setslot 8
@@ -889,6 +923,7 @@
 (rewrite 'f64vector-length 2 1 "C_u_i_64vector_length" #f #f)
 
 (rewrite 'not-pair? 17 1 "C_i_not_pair_p")
+(rewrite 'atom? 17 1 "C_i_not_pair_p")
 (rewrite 'null-list? 17 1 "C_i_null_list_p" "C_i_nullp")
 
 (rewrite 'u8vector->byte-vector 7 1 "C_slot" 1 #f)
@@ -934,6 +969,7 @@
 
 (let ()
   (define (rewrite-call/cc db classargs cont callargs)
+    ;; (call/cc <var>), <var> = (lambda (kont k) ... k is never used ...) -> (<var> #f)
     (and (= 1 (length callargs))
 	 (let ([val (first callargs)])
 	   (and (eq? '##core#variable (node-class val))
@@ -947,11 +983,9 @@
 			    (let ([var (or rest (second llist))])
 			      (and (not (get db var 'references))
 				   (not (get db var 'assigned)) 
-				   (begin
-				     (debugging 'x "removing unused `call/cc'")
-				     (make-node
-				      '##core#call '(#t)
-				      (list val cont (qnode #f)) ) ) ) ) ) ) ) ) ) ) ) ) )
+				   (make-node
+				    '##core#call '(#t)
+				    (list val cont (qnode #f)) ) ) ) ) ) ) ) ) ) ) ) )
   (rewrite 'call-with-current-continuation 8 rewrite-call/cc)
   (rewrite 'call/cc 8 rewrite-call/cc) )
 
