@@ -1,6 +1,6 @@
 /* chicken.h - General headerfile for compiler generated executables
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -39,8 +39,9 @@
 #ifndef ___CHICKEN
 #define ___CHICKEN
 
-#if (defined(HAVE_CONFIG_H) || defined(HAVE_CHICKEN_CONFIG_H)) && !defined(_MSC_VER) 
-# include <chicken-config.h>
+#if defined(HAVE_CONFIG_H) || defined(HAVE_CHICKEN_CONFIG_H)
+# include "chicken-config.h"
+# include "chicken-defaults.h"
 #endif
 
 #if !defined(__GNUC__) && !defined(__WATCOMC__)
@@ -48,7 +49,7 @@
 #  include <alloca.h>
 # else
 #  ifdef _AIX
-# pragma alloca
+#   pragma alloca
 #  else
 #   ifndef alloca /* predefined by HP cc +Olibcalls */
 char *alloca ();
@@ -180,10 +181,6 @@ char *alloca ();
 # define C_NONUNIX
 #endif
 
-#ifdef C_MICROSOFT_WINDOWS
-# include <windows.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -218,13 +215,18 @@ int strncasecmp(const char *one, const char *two, size_t n);
 
 #ifdef _MSC_VER
 # include <malloc.h>
+# include <io.h>
 # define alloca            _alloca
 # define strncasecmp       strnicmp
 # define isatty            _isatty
 # pragma warning(disable: 4101)
 #endif
 
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__NetBSD__)
+#ifdef __MINGW32__
+# include <malloc.h>
+#endif
+
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
 # define C_GNU_ENV
 #endif
 
@@ -252,6 +254,8 @@ int strncasecmp(const char *one, const char *two, size_t n);
 #define ___pointer
 #define ___u32              C_u32
 #define ___s32              C_s32
+#define ___u64              C_u64
+#define ___s64              C_s64
 
 
 /* Constants: */
@@ -356,7 +360,11 @@ int strncasecmp(const char *one, const char *two, size_t n);
 #define C_SIZEOF_LIST(n)          ((n) * 3 + 1)
 #define C_SIZEOF_PAIR             3
 #define C_SIZEOF_STRING(n)        (C_bytestowords(n) + 2)
-#define C_SIZEOF_SYMBOL           3
+#ifdef C_EXTRA_SYMBOL_SLOT
+# define C_SIZEOF_SYMBOL          4
+#else
+# define C_SIZEOF_SYMBOL          3
+#endif
 #define C_SIZEOF_INTERNED_SYMBOL(n) (C_SIZEOF_SYMBOL + C_SIZEOF_BUCKET + C_SIZEOF_STRING(n))
 #define C_SIZEOF_FLONUM           4
 #define C_SIZEOF_POINTER          2
@@ -384,14 +392,18 @@ int strncasecmp(const char *one, const char *two, size_t n);
 # define C_word                   long
 # define C_u32                    uint32_t
 # define C_s32                    int32_t
-# define C_u64                    unsigned long
-# define C_s64                    long
 #else
 # define C_word                   int
 # define C_u32                    unsigned int
 # define C_s32                    int
 #endif
 
+#if defined(_MSC_VER) || defined (__MINGW32__)
+# define C_s64                    __int64
+#else
+# define C_s64                    int64_t
+#endif
+
 #define C_char                    char
 #define C_uchar                   unsigned C_char
 #define C_byte                    char
@@ -443,7 +455,7 @@ int strncasecmp(const char *one, const char *two, size_t n);
 #define CHICKEN_global_ref(root)       C_u_i_car(((C_GC_ROOT *)(root))->value)
 #define CHICKEN_global_set(root, x)    C_mutate(&C_u_i_car(((C_GC_ROOT *)(root))->value), (x))
 
-#define CHICKEN_default_toplevel       C_default_stub_toplevel
+#define CHICKEN_default_toplevel       ((void *)C_default_stub_toplevel)
 
 
 /* Language specifics: */
@@ -679,6 +691,8 @@ DECL_C_PROC_p0 (128,  1,0,0,0,0,0,0,0)
 # include "chicken-libc-stubs.h"
 #endif
 
+#define C_return(x)                return(x)
+
 #define C_block_header(x)          (((C_SCHEME_BLOCK *)(x))->header)
 #define C_header_bits(x)           (C_block_header(x) & C_HEADER_BITS_MASK)
 #define C_header_size(x)           (C_block_header(x) & C_HEADER_SIZE_MASK)
@@ -686,7 +700,7 @@ DECL_C_PROC_p0 (128,  1,0,0,0,0,0,0,0)
 #define C_symbol_value(x)          (C_block_item(x, 0))
 #define C_block_item(x, i)         (((C_SCHEME_BLOCK *)(x))->data[ i ])
 #define C_set_block_item(x, i, y)  (C_block_item(x, i) = (y))
-#define C_save(x)		           (*(--C_temporary_stack) = (C_word)(x))
+#define C_save(x)	           (*(--C_temporary_stack) = (C_word)(x))
 #define C_adjust_stack(n)          (C_temporary_stack -= (n))
 #define C_rescue(x, i)             (C_temporary_stack[ i ] = (x))
 #define C_save_rest(s, c, n)  	   for(va_start(v, s); c-- > (n); C_save(va_arg(v, C_word)))
@@ -1085,6 +1099,7 @@ C_fctexport void *CHICKEN_new_gc_root();
 C_fctexport void CHICKEN_delete_gc_root(void *root);
 C_fctexport void *CHICKEN_global_lookup(char *name);
 C_fctexport int CHICKEN_is_running();
+C_fctexport void CHICKEN_interrupt();
 
 C_fctexport void C_check_nursery_minimum(C_word size);
 C_fctexport int C_fcall C_save_callback_continuation(C_word **ptr, C_word k);
@@ -1099,6 +1114,7 @@ C_fctexport C_word C_fcall C_disable_interrupts(void) C_regparm;
 C_fctexport void C_fcall C_paranoid_check_for_interrupt(void) C_regparm;
 C_fctexport double C_fcall C_c_double(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_num_to_int(C_word x) C_regparm;
+C_fctexport C_s64 C_fcall C_num_to_int64(C_word x) C_regparm;
 C_fctexport C_uword C_fcall C_num_to_unsigned_int(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_int_to_num(C_word **ptr, C_word n) C_regparm;
 C_fctexport C_word C_fcall C_unsigned_int_to_num(C_word **ptr, C_uword n) C_regparm;
@@ -1113,6 +1129,7 @@ C_fctexport char *C_fcall C_string_or_null(C_word x) C_regparm;
 C_fctexport void *C_fcall C_data_pointer_or_null(C_word x) C_regparm;
 C_fctexport void *C_fcall C_srfi_4_vector_or_null(C_word x) C_regparm;
 C_fctexport void *C_fcall C_c_pointer_or_null(C_word x) C_regparm;
+C_fctexport void *C_fcall C_scheme_or_c_pointer(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_flonum_in_fixnum_range_p(C_word n) C_regparm;
 C_fctexport void C_zap_strings(C_word str);
 C_fctexport void C_set_or_change_heap_size(C_word heap, int reintern);
@@ -1177,6 +1194,7 @@ C_fctexport void *C_fcall C_retrieve_proc(C_word closure) C_regparm;
 C_fctexport C_word C_fcall C_permanentp(C_word x) C_regparm;
 C_fctexport int C_in_stackp(C_word x) C_regparm;
 C_fctexport int C_fcall C_in_heapp(C_word x) C_regparm;
+C_fctexport int C_fcall C_in_fromspacep(C_word x) C_regparm;
 C_fctexport void C_fcall C_trace(C_char *name) C_regparm;
 C_fctexport C_word C_fcall C_emit_trace_info(C_word x, C_word y, C_word t) C_regparm;
 C_fctexport C_word C_fcall C_hash_string(C_word str) C_regparm;
@@ -1212,6 +1230,7 @@ C_fctexport C_word C_fcall C_lookup_symbol(C_word sym) C_regparm;
 C_fctexport C_word C_enumerate_symbols(C_SYMBOL_TABLE *stable, C_word pos) C_regparm;
 C_fctexport void C_do_register_finalizer(C_word x, C_word proc);
 C_fctexport int C_do_unregister_finalizer(C_word x);
+C_fctexport C_word C_dbg_hook(C_word x);
 
 C_fctimport void C_ccall C_toplevel(C_word c, C_word self, C_word k) C_noret;
 C_fctexport void C_ccall C_stop_timer(C_word c, C_word closure, C_word k) C_noret;
@@ -1291,13 +1310,14 @@ C_fctexport C_word C_fcall C_a_i_bytevector(C_word **a, int c, C_word x) C_regpa
 C_fctexport C_word C_fcall C_i_eqvp(C_word x, C_word y) C_regparm;
 C_fctexport C_word C_fcall C_i_symbolp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_pairp(C_word x) C_regparm;
-C_fctexport C_word C_fcall C_i_atomp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_vectorp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_closurep(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_portp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_stringp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_numberp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_integerp(C_word x) C_regparm;
+C_fctexport C_word C_fcall C_i_flonump(C_word x) C_regparm;
+C_fctexport C_word C_fcall C_i_finitep(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_locativep(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_fixnum_min(C_word x, C_word y) C_regparm;
 C_fctexport C_word C_fcall C_i_fixnum_max(C_word x, C_word y) C_regparm;
@@ -1408,6 +1428,7 @@ C_fctexport C_word C_fcall C_i_foreign_string_argumentp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_foreign_symbol_argumentp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_foreign_tagged_pointer_argumentp(C_word x, C_word t) C_regparm;
 C_fctexport C_word C_fcall C_i_foreign_pointer_argumentp(C_word x) C_regparm;
+C_fctexport C_word C_fcall C_i_foreign_scheme_or_c_pointer_argumentp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_foreign_integer_argumentp(C_word x) C_regparm;
 C_fctexport C_word C_fcall C_i_foreign_unsigned_integer_argumentp(C_word x) C_regparm;
 
