@@ -1,6 +1,6 @@
 ;;;; chicken.scm - The CHICKEN Scheme compiler (loader/main-module)
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -34,8 +34,7 @@
 
 
 (declare
-  (uses extras srfi-1 match-support srfi-4 utils support compiler optimizer driver platform backend 
-	easyffi partition lolevel)
+  (uses extras srfi-1 match srfi-4 utils support compiler optimizer driver platform backend)
   (run-time-macros) )
 
 
@@ -45,21 +44,21 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home optimization-iterations process-command-line
-  foreign-type-table-size file-io-only nonwinding-call/cc debugging
+  file-io-only nonwinding-call/cc debugging
   unit-name insert-timer-checks used-units zap-strings-flag
-  foreign-declarations debugging-executable block-compilation analysis-database-size line-number-database-size
-  target-heap-size target-stack-size try-harder default-installation-home 
+  foreign-declarations debugging-executable block-compilation line-number-database-size
+  target-heap-size target-stack-size 
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  broken-constant-nodes inline-substitutions-enabled
+  broken-constant-nodes inline-substitutions-enabled compiler-warning
   direct-call-ids foreign-type-table first-analysis
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
   reorganize-recursive-bindings substitution-table simplify-named-call
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas!
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -78,8 +77,7 @@
 
 (eval-when (load) 
   (include "chicken-more-macros")
-  (include "chicken-ffi-macros")
-  (include "chicken-match-macros") )
+  (include "chicken-ffi-macros") )
 
 (##sys#provide 'extras 'srfi-1 'srfi-4)
 
@@ -133,7 +131,7 @@
 		   [(3)
 		    (set! options
 		      (cons* 'optimize-leaf-routines 'unsafe options) ) ]
-		   [else (warning "invalid optimization level ~S - ignored" (car rest))] )
+		   [else (compiler-warning 'usage "invalid optimization level ~S - ignored" (car rest))] )
 		 (loop (cdr rest)) ) ]
 	      [(eq? 'debug-level o)
 	       (let ([level (string->number (car rest))])
@@ -141,7 +139,7 @@
 		   [(0) (set! options (cons* 'no-lambda-info 'no-trace options))]
 		   [(1) (set! options (cons 'no-trace options))]
 		   [(2) #f]
-		   [else (warning "invalid debug level ~S - ignored" (car rest))] )
+		   [else (compiler-warning 'usage "invalid debug level ~S - ignored" (car rest))] )
 		 (loop (cdr rest)) ) ]
 	      [(eq? 'benchmark-mode o)
 	       (set! options 
@@ -155,7 +153,9 @@
 		   (loop (cdr rest))
 		   (quit "missing argument to `-~s' option" o) ) ]
 	      [else
-	       (warning "invalid compiler option `-~S' - ignored" o)
+	       (compiler-warning 
+		'usage "invalid compiler option `~a' - ignored" 
+		(if (string? o) o (conc "-" o)) )
 	       (loop rest) ] ) ) ) )
   (apply compile-source-file filename options)
   (exit) )
