@@ -5,7 +5,7 @@
 ;
 ;
 ;-----------------------------------------------------------------------------------------------------------
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -55,10 +55,12 @@
 ; (no-bound-checks)
 ; (no-argc-checks)
 ; (no-procedure-checks)
+; (no-procedure-checks-for-usual-bindings)
 ; (block-global {<name>})
 ; (lambda-lift)
 ; (hide {<name>})
 ; (disable-interrupts)
+; (disable-warning <class> ...)
 ; (always-bound {<name>})
 ; (foreign-declare {<string>})
 ; (foreign-parse {<string>})
@@ -72,6 +74,9 @@
 ; (custom-declare (<tag> <name> <filename> <arg> ...) <string> ...)
 ; (data <tag1> <exp1> ...)
 ; (post-process <string> ...)
+; (emit-exports <string>)
+; (keep-shadowed-macros)
+; (import <symbol-or-string> ...)
 ;
 ;   <type> = fixnum | generic
 ;
@@ -228,11 +233,11 @@
 
 (declare
  (unit compiler)
- (foreign-declare #<<EOF
-#ifdef C_USE_C_DEFAULTS
-# include "chicken-defaults.h"
-#else
-# define C_INSTALL_HOME NULL
+ (disable-warning var) )
+
+#>
+#ifndef C_USE_C_DEFAULTS
+# define C_INSTALL_SHARE_HOME NULL
 #endif
 
 #ifndef C_DEFAULT_TARGET_STACK_SIZE
@@ -242,8 +247,7 @@
 #ifndef C_DEFAULT_TARGET_HEAP_SIZE
 # define C_DEFAULT_TARGET_HEAP_SIZE 0
 #endif
-EOF
-) )
+<#
 
 
 #{compiler
@@ -251,28 +255,29 @@ EOF
   default-standard-bindings default-extended-bindings side-effecting-standard-bindings
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
-  installation-home decompose-lambda-list external-to-pointer defconstant-bindings
-  foreign-type-table-size copy-node! error-is-extended-binding toplevel-scope toplevel-lambda-id
+  installation-home decompose-lambda-list external-to-pointer defconstant-bindings constant-declarations
+  copy-node! error-is-extended-binding toplevel-scope toplevel-lambda-id
   unit-name insert-timer-checks used-units external-variables require-imports-flag custom-declare-alist
   profile-info-vector-name finish-foreign-result pending-canonicalizations
-  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
+  foreign-declarations emit-trace-info block-compilation line-number-database-size
   always-bound-to-procedure block-globals make-block-variable-literal block-variable-literal? block-variable-literal-name
-  target-heap-size target-stack-size try-harder default-installation-home valid-c-identifier?
-  target-initial-heap-size internal-bindings source-filename dump-nodes
+  target-heap-size target-stack-size valid-c-identifier?
+  target-initial-heap-size internal-bindings source-filename dump-nodes source-info->string
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
   broken-constant-nodes inline-substitutions-enabled loop-lambda-names expand-profile-lambda
-  profile-lambda-list profile-lambda-index emit-profile expand-profile-lambda no-c-syntax-checks
-  direct-call-ids foreign-type-table first-analysis callback-names namespace-table
+  profile-lambda-list profile-lambda-index emit-profile expand-profile-lambda
+  direct-call-ids foreign-type-table first-analysis callback-names namespace-table disabled-warnings
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database! scan-toplevel-assignments
+  compiler-warning import-table use-import-table
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
   reorganize-recursive-bindings substitution-table simplify-named-call inline-max-size
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub 
   expand-foreign-lambda* data-declarations emit-control-file-item expand-foreign-primitive
-  factor partition-fm process-declaration external-protos-first basic-literal? emit-line-info
+  process-declaration external-protos-first basic-literal? emit-line-info
   transform-direct-lambdas! expand-foreign-callback-lambda* debugging emit-unsafe-marker
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -280,22 +285,24 @@ EOF
   simple-lambda-node? compute-database-statistics print-program-statistics output gen gen-list 
   pprint-expressions-to-file foreign-type-check estimate-foreign-result-size scan-used-variables scan-free-variables
   topological-sort print-version print-usage initialize-analysis-database export-list csc-control-file
-  estimate-foreign-result-location-size compressed-literals-initializer parse-easy-ffi
+  estimate-foreign-result-location-size compressed-literals-initializer
   expand-foreign-callback-lambda default-optimization-passes default-optimization-passes-when-trying-harder
   units-used-by-default words-per-flonum disable-stack-overflow-checking
-  parameter-limit eq-inline-operator optimizable-rest-argument-operators check-c-syntax postponed-initforms
+  parameter-limit eq-inline-operator optimizable-rest-argument-operators postponed-initforms
   membership-test-operators membership-unfold-limit valid-compiler-options valid-compiler-options-with-argument
   make-random-name final-foreign-type real-name-table real-name set-real-name! safe-globals-flag
   location-pointer-map literal-compression-threshold compressed-literals compressable-literal
+  lookup-exports-file undefine-shadowed-macros
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
-  process-custom-declaration do-lambda-lifting file-requirements emit-closure-info
+  process-custom-declaration do-lambda-lifting file-requirements emit-closure-info export-file-name
   foreign-argument-conversion foreign-result-conversion foreign-type-convert-argument foreign-type-convert-result}
 
 (eval-when (compile eval)
   (match-error-control #:fail) )
 
+
 (include "tweaks")
-(include "parameters")
+
 
 (define-inline (gensym-f-id) (gensym 'f_))
 
@@ -305,7 +312,7 @@ EOF
   (define default-target-stack-size #f) )
 
 (eval-when (load)
-  (define-foreign-variable installation-home c-string "C_INSTALL_HOME")
+  (define-foreign-variable installation-home c-string "C_INSTALL_SHARE_HOME")
   (define-foreign-variable default-target-heap-size int "C_DEFAULT_TARGET_HEAP_SIZE")
   (define-foreign-variable default-target-stack-size int "C_DEFAULT_TARGET_STACK_SIZE") )
 
@@ -316,6 +323,16 @@ EOF
 (define user-pass-2 (make-parameter #f))
 (define user-post-analysis-pass (make-parameter #f))
 
+(define-constant foreign-type-table-size 301)
+(define-constant analysis-database-size 3001)
+(define-constant default-line-number-database-size 997)
+(define-constant inline-table-size 301)
+(define-constant constant-table-size 301)
+(define-constant real-name-table-size 997)
+(define-constant import-table-size 997)
+(define-constant default-literal-compression-threshold 50)
+(define-constant default-inline-max-size 10)
+
 
 ;;; Global variables containing compilation parameters:
 
@@ -335,7 +352,6 @@ EOF
 (define target-heap-size #f)
 (define target-initial-heap-size #f)
 (define target-stack-size #f)
-(define try-harder #f)
 (define optimize-leaf-routines #f)
 (define emit-profile #f)
 (define no-bound-checks #f)
@@ -352,18 +368,21 @@ EOF
 (define disable-stack-overflow-checking #f)
 (define namespace-table '())
 (define require-imports-flag #f)
-(define no-c-syntax-checks #f)
 (define emit-unsafe-marker #f)
 (define external-protos-first #f)
 (define do-lambda-lifting #f)
 (define inline-max-size -1)
 (define emit-closure-info #t)
 (define emit-line-info #f)
+(define export-file-name #f)
+(define import-table #f)
+(define use-import-table #f)
+(define undefine-shadowed-macros #t)
+(define constant-declarations '())
 
 
 ;;; These are here so that the backend can access them:
 
-(define default-installation-home installation-home)
 (define default-default-target-heap-size default-target-heap-size)
 (define default-default-target-stack-size default-target-stack-size)
 
@@ -427,6 +446,9 @@ EOF
   (set! profile-info-vector-name (make-random-name 'profile-info))
   (set! real-name-table (make-vector real-name-table-size '()))
   (set! file-requirements (make-hash-table eq?))
+  (if import-table
+      (vector-fill! import-table '())
+      (set! import-table (make-vector import-table-size '())) )
   (if foreign-type-table
       (vector-fill! foreign-type-table '())
       (set! foreign-type-table (make-vector foreign-type-table-size '())) ) )
@@ -564,12 +586,23 @@ EOF
 				'(##core#undefined)
 				(let ([id (cadar ids)])
 				  (let-values ([(exp f) (##sys#do-the-right-thing id #t)])
-				    (unless (or f 
-						(and (symbol? id)
-						     (or (test-feature? id)
+				    (if (not (or f 
+						 (and (symbol? id)
+						     (or (feature? id)
 							 (##sys#find-extension
-							  (##sys#canonicalize-extension-path id 'require-extension) #f)) ) )
-				      (warning "extension `~A' is currently not installed" id) )
+							  (##sys#canonicalize-extension-path 
+							   id 'require-extension) #f)) ) ) )
+					(compiler-warning 
+					 'ext "extension `~A' is currently not installed" id)
+					(unless (and-let* (use-import-table
+							   ((symbol? id))
+							   (info (##sys#extension-information id #f)) 
+							   (exps (assq 'exports info)) )
+						  (for-each
+						   (cut ##sys#hash-table-set! import-table <> id)
+						   (cdr exps) )
+						  #t)
+					  (lookup-exports-file id) ) )
 				    `(begin ,exp ,(loop (cdr ids))) ) ) ) )
 			  ae me dest) )
 
@@ -639,15 +672,18 @@ EOF
 				[ln (get-line x)]
 				[val (walk (caddr x) ae me var0)] )
 			   (when (and safe-globals-flag (eq? var var0))
-			     (set! always-bound-to-procedure (lset-adjoin eq? always-bound-to-procedure var))
+			     (set! always-bound-to-procedure
+			       (lset-adjoin eq? always-bound-to-procedure var))
 			     (set! always-bound (lset-adjoin eq? always-bound var)) )
 			   (when (eq? var var0)
 			     (when (macro? var)
-			       (warning "assigned global variable `~S' is a macro ~A"
-					var
-					(if ln (sprintf "in line ~S" ln) "") ) ) )
+			       (compiler-warning 
+				'var "assigned global variable `~S' is a macro ~A"
+				var
+				(if ln (sprintf "in line ~S" ln) "") )
+			       (when undefine-shadowed-macros (undefine-macro! var) ) ) )
 			   (when (keyword? var)
-			     (warning "assignment to keyword `~S'" var) )
+			     (compiler-warning 'syntax "assignment to keyword `~S'" var) )
 			   (cond [(assq var foreign-variables)
 				  => (lambda (fv)
 				       (let ([type (second fv)]
@@ -726,7 +762,6 @@ EOF
 				[name (if (pair? (cdddr x))
 					  (cadr (fourth x))
 					  (symbol->string var) ) ] )
-			   (check-c-syntax name 'define-foreign-variable)
 			   (set! foreign-variables
 			     (cons (list var type (if (string? name) name (symbol->string name)))
 				   foreign-variables))
@@ -815,8 +850,11 @@ EOF
 				  '(##core#undefined) ]
 				 [else
 				  (unless (basic-literal? val)
-				    (warning "value for constant binding appears not to be a valid literal: ~s" val) )
-				  (let ([var (gensym)])
+				    (compiler-warning 
+				     'const
+				     "value for constant binding appears not to be a valid literal: ~s"
+				     val) )
+				  (let ([var (gensym "constant")])
 				    (##sys#hash-table-set! constant-table name (list var))
 				    (set! mutable-constants (alist-cons var val mutable-constants))
 				    (set! block-globals (cons var block-globals))
@@ -865,9 +903,17 @@ EOF
 						       (loop (cdr vars) (cdr types)) ) ) ) )
 					  ,(foreign-type-convert-argument
 					    `(let ()
-					       ,@(if (memq rtype '(nonnull-c-string c-string))
-						     `((##sys#make-c-string (let () ,@(cddr lam))))
-						     (cddr lam)) )
+					       ,@(match rtype
+						   ((or '(const nonnull-c-string) 'nonnull-c-string)
+						    `((##sys#make-c-string (let () ,@(cddr lam)))))
+						   ((or '(const c-string*) 'c-string*)
+						    (##sys#syntax-error-hook
+						     "`c-string*' is not a valid resulte type for callback procedures"
+						     name) )
+						   ((or 'c-string '(const c-string))
+						    `((let ((r (let () ,@(cddr lam))))
+							(and r (##sys#make-c-string r)) ) ) )
+						   (_ (cddr lam)) ) )
 					    rtype) ) )
 				      ae me #f) ) ) ) )
 
@@ -907,7 +953,7 @@ EOF
 	   (quit "syntax error - malformed expression `~s'" x) )
 
 	  ((constant? (car x))
-	   (warning "literal in operator position: ~S" x) 
+	   (compiler-warning 'syntax "literal in operator position: ~S" x) 
 	   (mapwalk x ae me) )
 
 	  ((and (pair? (car x)) (eq? 'lambda (caar x)))
@@ -954,6 +1000,8 @@ EOF
        ((uses)
 	(let ([us (cdr spec)])
 	  (apply register-feature! us)
+	  (when use-import-table
+	    (for-each lookup-exports-file us) )
 	  (when (pair? us)
 	    (hash-table-update! file-requirements 'uses (cut lset-union eq? us <>) (lambda () us))
 	    (let ((units (map (lambda (u) (string->c-identifier (stringify u))) us)))
@@ -964,7 +1012,7 @@ EOF
 	       [un (string->c-identifier (stringify u))] )
 	  (hash-table-set! file-requirements 'unit u)
 	  (when (and unit-name (not (string=? unit-name un)))
-	    (warning "unit was already given a name (new name is ignored)") )
+	    (compiler-warning 'usage "unit was already given a name (new name is ignored)") )
 	  (set! unit-name un) ) )
        ((standard-bindings)
 	(if (null? (cdr spec))
@@ -996,8 +1044,14 @@ EOF
        ((no-procedure-checks) (set! no-procedure-checks #t))
        ((interrupts-enabled) (set! insert-timer-checks #t))
        ((disable-interrupts) (set! insert-timer-checks #f))
+       ((disable-warning) (set! disabled-warnings (append (cdr spec) disabled-warnings)))
        ((always-bound) (set! always-bound (append (cdr spec) always-bound)))
        ((safe-globals) (set! safe-globals-flag #t))
+       ((no-procedure-checks-for-usual-bindings)
+	(set! always-bound-to-procedure
+	  (append default-standard-bindings default-extended-bindings always-bound-to-procedure))
+	(set! always-bound
+	  (append default-standard-bindings default-extended-bindings always-bound)) )
        ((bound-to-procedure)
 	(set! always-bound-to-procedure (append (cdr spec) always-bound-to-procedure))
 	(set! always-bound (append (cdr spec) always-bound)) )
@@ -1005,14 +1059,6 @@ EOF
 	(let ([fds (cdr spec)])
 	  (if (every string? fds)
 	      (set! foreign-declarations (append foreign-declarations fds))
-	      (quit "syntax error in declaration: `~S'" spec) )
-	  (check-c-syntax (apply string-append fds) 'foreign-declare) ) )
-       ((foreign-parse)
-	(let ([fds (cdr spec)])
-	  (if (every string? fds)
-	      (return 
-	       (##sys#compiler-toplevel-macroexpand-hook 
-		`(begin ,@(parse-easy-ffi (string-intersperse fds ""))) ) )
 	      (quit "syntax error in declaration: `~S'" spec) ) ) )
        ((custom-declare)
 	(if (or (not (list? spec)) (not (list? (cadr spec))) (< (length (cadr spec)) 3))
@@ -1028,7 +1074,7 @@ EOF
 	   `(post-process ,@(map (cut string-substitute "\\$@" file <>) (cdr spec))) ) ) )
        ((block) (set! block-compilation #t))
        ((separate) (set! block-compilation #f))
-       ((check-c-syntax) (set! no-c-syntax-checks #f))
+       ((keep-shadowed-macros) (set! undefine-shadowed-macros #f))
        ((not)
 	(check-decl spec 1)
 	(case (second spec)
@@ -1056,11 +1102,10 @@ EOF
 	   (check-decl spec 1 1)
 	   (case (cadr spec)
 	     [(interrupts-enabled) (set! insert-timer-checks #f)]
-	     [(check-c-syntax) (set! no-c-syntax-checks #t)]
 	     [(safe) 
 	      (set! unsafe #t)
 	      (##match#set-error-control #:unspecified) ]
-	     [else (warning "illegal declaration specifier `~s'" spec)] ) ] ) )
+	     [else (compiler-warning 'syntax "illegal declaration specifier `~s'" spec)] ) ] ) )
        ((run-time-macros) (set! ##sys#enable-runtime-macros #t))
        ((block-global hide) 
 	(let ([syms (cdr spec)])
@@ -1077,6 +1122,10 @@ EOF
 	      default-literal-compression-threshold) ) 
 	(when (and (list? spec) (= 3 (length spec)))
 	  (set! compressed-literals-initializer (third spec)) ) )
+       ((emit-exports)
+	(if (null? (cdr spec))
+	    (quit "invalid `emit-exports' declaration" spec)
+	    (set! export-file-name (cadr spec)) ) )
        ((emit-external-prototypes-first)
 	(set! external-protos-first #t) )
        ((lambda-lift) (set! do-lambda-lifting #t))
@@ -1101,7 +1150,23 @@ EOF
 		(set! namespace-table 
 		  (alist-update! ns (lset-union eq? oldsyms (cdr syms)) namespace-table eq?) ) )
 	      (quit "invalid arguments to `namespace' declaration: ~S" spec) ) ) )
-       (else (warning "illegal declaration specifier `~s'" spec)) )
+       ((constant)
+	(let ((syms (cdr spec)))
+	  (if (every symbol? syms)
+	      (set! constant-declarations (append syms constant-declarations))
+	      (quit "invalid arguments to `constant' declaration: ~S" spec)) ) )
+       ((import)
+	(let-values (((syms strs) 
+		      (partition
+		       (lambda (x)
+			 (cond ((symbol? x) #t)
+			       ((string? x) #f)
+			       (else (quit "argument to `import' declaration is not a string or symbol" x)) ) )
+		       (cdr spec) ) ) )
+	  (set! use-import-table #t)
+	  (for-each (cut ##sys#hash-table-set! import-table <> "<here>") syms)
+	  (for-each lookup-exports-file strs) ) )
+       (else (compiler-warning 'syntax "illegal declaration specifier `~s'" spec)) )
      '(##core#undefined) ) ) )
 
 
@@ -1167,7 +1232,6 @@ EOF
 	 [body (apply string-append (map cadr (cdddr exp)))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
-    (check-c-syntax body 'foreign-lambda*)
     (create-foreign-stub rtype #f argtypes argnames body #f #f) ) )
 
 (define (expand-foreign-callback-lambda* exp)
@@ -1176,7 +1240,6 @@ EOF
 	 [body (apply string-append (map cadr (cdddr exp)))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
-    (check-c-syntax body 'foreign-callback-lambda*)
     (create-foreign-stub rtype #f argtypes argnames body #t #t) ) )
 
 (define (expand-foreign-primitive exp)
@@ -1186,7 +1249,6 @@ EOF
 	 [body (apply string-append (map cadr (if hasrtype (cdddr exp) (cddr exp))))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
-    (check-c-syntax body 'foreign-primitive)
     (create-foreign-stub rtype #f argtypes argnames body #f #t) ) )
 
 
@@ -1449,9 +1511,9 @@ EOF
 		  [val (car subs)] )
 	     (when first-analysis 
 	       (cond [(get db var 'standard-binding)
-		      (warning "redefinition of standard binding `~S'" var) ]
+		      (compiler-warning 'redef "redefinition of standard binding `~S'" var) ]
 		     [(get db var 'extended-binding)
-		      (warning "redefinition of extended binding `~S'" var) ] )
+		      (compiler-warning 'redef "redefinition of extended binding `~S'" var) ] )
 	       (put! db var 'potential-value val) )
 	     (when (and (not (memq var localenv)) 
 			(not (memq var env)) )
@@ -1484,9 +1546,8 @@ EOF
 	    ((or block-compilation
 		 (memq var env)
 		 (get db var 'constant)
-		 ;;(memq var inline-list)       - would be nice, but the might be customized...
-		 (and try-harder 
-		      (memq var block-globals) ) )
+		 ;;(memq var inline-list)       - would be nice, but might be customized...
+		 (memq var block-globals) )
 	     (let ((props (get-all db var 'unknown 'value))
 		   (home (get db var 'home)) )
 	       (unless (assq 'unknown props)
@@ -1572,10 +1633,11 @@ EOF
 	 ;;  in block mode, then issue warning:
 	 (when (and first-analysis global (null? references))
 	   (when assigned-locally
-	     (warning "local assignment to unused variable `~S' may be unintended" sym) )
-	   (when (or block-compilation
-		     (and export-list (not (memq sym export-list))) )
-	     (warning "global variable `~S' is never used" sym) ) )
+	     (compiler-warning 'var "local assignment to unused variable `~S' may be unintended" sym) )
+	   (when (and (or block-compilation
+			  (and export-list (not (memq sym export-list))) )
+		      (not (assq sym mutable-constants)) )
+	     (compiler-warning 'var "global variable `~S' is never used" sym) ) )
 
  	 ;; Make 'boxed, if 'assigned & 'captured:
 	 (when (and assigned captured)
@@ -1583,7 +1645,7 @@ EOF
 
 	 ;; Make 'contractable, if it has a procedure as known value, has only one use and one call-site and
 	 ;;  if the lambda has no free non-global variables or is an internal lambda. Make 'inlinable if
-	 ;;  use/call count is > 1:
+	 ;;  use/call count is 1:
 	 (when value
 	   (let ((valparams (node-parameters value)))
 	     (when (and (eq? '##core#lambda (node-class value))
@@ -1614,7 +1676,7 @@ EOF
 	     (when (and (eq? '##core#lambda (node-class value))
 			(= nreferences ncall-sites) )
 	       (let ([lparams (node-parameters value)])
-		 (when (and (second lparams) try-harder)
+		 (when (second lparams)
 		   (decompose-lambda-list
 		    (third lparams)
 		    (lambda (vars argc rest)
@@ -1630,7 +1692,7 @@ EOF
 			 vars) )
 		      (cond [(and has (not (memq sym callback-names)))
 			     (put! db (first lparams) 'has-unused-parameters #t) ]
-			    [(and try-harder rest)
+			    [rest
 			     (set! explicitly-consed (cons rest explicitly-consed))
 			     (put! db (first lparams) 'explicit-rest #t) ] ) ) ) ) ) ) ) )
 
@@ -1692,8 +1754,7 @@ EOF
 	 ;;  rest-argument operators:
 	 ;; - Add variable to "rest-parameters-promoted-to-vector", because subsequent optimization will
 	 ;;   change variables context (operators applied to it).
-	 (when (and try-harder
-		    rest-parameter
+	 (when (and rest-parameter
 		    (not assigned)
 		    (= nreferences o-r/access-count) )
 	   (set! rest-parameters-promoted-to-vector (lset-adjoin eq? rest-parameters-promoted-to-vector sym))
@@ -1766,7 +1827,10 @@ EOF
 						     (= (length refs) (length sites)) 
 						     (proper-list? llist) ) ] )
 					  (when (and name custom (not (= (length llist) (length (cdr subs)))))
-					    (warning "known procedure called with wrong number of arguments: `~A'" name) )
+					    (compiler-warning
+					     'call 
+					     "known procedure called with wrong number of arguments: ~A" 
+					     (source-info->string name) ) )
 					  (register-direct-call! id)
 					  (when custom (register-customizable! varname id)) 
 					  (list id custom) )
@@ -1979,10 +2043,9 @@ EOF
   customizable				; boolean
   rest-argument-mode			; #f | LIST | VECTOR | UNUSED
   body					; expression
-  direct				; boolean
-  partition)                            ; integer
+  direct)				; boolean
   
-(define (prepare-for-code-generation node db partitions)
+(define (prepare-for-code-generation node db)
   (let ([literals '()]
         [lambdas '()]
         [temporaries 0]
@@ -2119,8 +2182,7 @@ EOF
 			   (or direct (get db id 'customizable))
 			   rest-mode
 			   body
-			   direct
-                           0)
+			   direct)
 			  lambdas) )
 		  (set! looping lping)
 		  (set! temporaries temps)
@@ -2181,7 +2243,9 @@ EOF
 		   ((number? c)
 		    (cond ((eq? 'fixnum number-type)
 			   (cond ((integer? c)
-				  (warning "coerced inexact literal number `~S' to fixnum ~S" c (inexact->exact c))
+				  (compiler-warning 
+				   'type 
+				   "coerced inexact literal number `~S' to fixnum ~S" c (inexact->exact c))
 				  (immediate-literal (inexact->exact c)) )
 				 (else (quit "can not coerce inexact literal `~S' to fixnum" c)) ) )
 			  (else (make-node '##core#literal (list (literal c)) '())) ) )
@@ -2223,286 +2287,10 @@ EOF
 		       ((eof-object? x) '(eof))
 		       (else (bomb "bad immediate (prepare)")) )
 		 '() ) )
-
-    (define (partition-phase! partitions)
-      (define (set-inregion! cells partition)
-        (for-each
-          (lambda (c)
-            (graph-cell-info$$-inregion-set!
-              (graph-cell$$-info c)
-              (eq? (graph*partition c) partition)))
-          cells))
-      (let [(callgraph (make-graph$ eq?))]
-        (debugging 'p "partitioning phase...")
-        ;; init call graph
-        (for-each
-          (lambda (l)
-            (graph$-add-cell! callgraph (make-graph-cell$ (lambda-literal-id l))))
-          lambdas)
-        (##sys#hash-table-for-each
-        (lambda (sym plist)
-          (let loop [(es plist)]
-            (if (and (pair? es)
-                     (eq? (caar es) 'contains))
-                (for-each
-                  (lambda (o)
-                    (let [(symcell (graph$-get callgraph sym))]
-                      (and symcell ;; toplevel et al aren't in lambdas
-                           (graph-cell$-add-undirected-edge!
-                             symcell
-                             (graph$-get callgraph o)))))
-                  (cdar es)))
-            (and (pair? es) (loop (cdr es)))))
-        db)
-        ;; partition recursion.  i threw together something with
-        ;; primes to decompose any partition into bi-partitions.  i am
-        ;; sure there is a better way.
-        (randomize 20040619) ;; make partition repeatable
-	(define partition-level 0)
-	(define partition-index 0)
-	(let pway ([factors (sort (factor partitions) <)]
-		   [cells (graph$->list callgraph)])
-	  (cond-expand [testing 
-			(let loop ([n partition-level]) 
-			  (and (> n 0) (display "  ") (loop (sub1 n)))) ]
-		       [else] )
-	  (set! partition-level (add1 partition-level))
-	  (cond
-	   ;; terminate and update
-	   [(null? factors)
-	    (for-each
-	     (lambda (cell)
-	       (let* [(id (graph*id cell))
-		      (ll (member id lambdas
-				  (lambda (x ei)
-				    (eq? x (lambda-literal-id ei)))))]
-		 (and ll
-		      (lambda-literal-partition-set!
-		       (car ll)
-		       partition-index))))
-	     cells)
-	    (set! partition-index (add1 partition-index))]
-	   ;; just continue if 1
-	   [(= (car factors) 1)
-	    (pway (cdr factors) cells)]
-	   ;; bi-partition, balanced 50/50
-	   [(= (car factors) 2)
-	    (partition-fm cells
-			  graph*id graph*color graph*color-set!
-			  graph*partition graph*partition-move!
-			  graph*neighbours
-			  graph*gain
-			  (lambda () (graph*cost callgraph))
-			  (lambda (w n#f n#t) 
-			    (graph*balance cells
-					   (length cells)
-					   w n#f n#t))
-			  ;; 50/50
-			  '(1 . 1)
-			  10)
-
-	    (let ([bpart#f (map (complement graph*partition) cells)]
-		  [bpart#t (map graph*partition cells)])
-	      ;; descend binary tree
-              
-              (set-inregion! cells #f)
-	      (pway (cdr factors) (compress bpart#f cells))
-              
-              (set-inregion! cells #t)
-	      (pway (cdr factors) (compress bpart#t cells)))
-	    ]
-	   ;; bi-partition, but unbalanced
-	   [else
-	    (partition-fm cells
-			  graph*id graph*color graph*color-set!
-			  graph*partition graph*partition-move!
-			  graph*neighbours
-			  graph*gain
-			  (lambda () (graph*cost callgraph))
-			  (lambda (w n#f n#t) 
-			    (graph*balance cells
-					   (length cells)
-					   w n#f n#t))			  
-			  ;; split into 1:(prime-1) ratio
-			  (cons 1 (sub1 (car factors))) 
-			  10)
-	    (let ([bpart#f (map (complement graph*partition) cells)]
-		  [bpart#t (map graph*partition cells)])
-	      ;;; modify binary tree
-              
-              (set-inregion! cells #f)
-	      (pway (cons 1 (cdr factors)) 
-		    (compress bpart#f cells))
-              
-              (set-inregion! cells #t)
-	      (pway (append (factor (sub1 (car factors))) (cdr factors)) 
-		    (compress bpart#t cells)))
-	    ]
-	   )
-	  (set! partition-level (sub1 partition-level))
-	  )
-	(and #f ;; debugging
-	     (for-each
-	      (lambda (l)
-		(display (lambda-literal-id l))
-		(display " in ")
-		(display (lambda-literal-partition l))
-		(newline))
-	      lambdas))
-        (randomize);; reset random
-        ))
     
     (debugging 'p "preparation phase...")
-    (let* ((node2 (walk node '() #f '()))
-	   (t0 (##sys#fudge 6)) )
-      (partition-phase! partitions)
-      (debugging 'b (sprintf "  time needed for partitioning: ~A ms" (- (##sys#fudge 6) t0)))
+    (let ((node2 (walk node '() #f '())))
       (debugging 'o "fast box initializations" fastinits)
       (debugging 'o "fast global references" fastrefs)
       (debugging 'o "fast global assignments" fastsets)
       (values node2 literals lambdas) ) ) )
-
-
-;; factor - Jonah Beckford - slimmed down from SLIB. -*- Hen -*-
-
-;;;; "factor.scm" factorization, prime test and generation
-;;; Copyright (C) 1991, 1992, 1993, 1998 Aubrey Jaffer
-;
-;Permission to copy this software, to modify it, to redistribute it,
-;to distribute modified versions, and to use it for any purpose is
-;granted, subject to the following restrictions and understandings.
-;
-;1.  Any copy made of this software must include this copyright notice
-;in full.
-;
-;2.  I have made no warranty or representation that the operation of
-;this software will be error-free, and I am under no obligation to
-;provide any services, by way of maintenance, update, or otherwise.
-;
-;3.  In conjunction with products arising from the use of this
-;material, there shall be no use of my name in any advertising,
-;promotional, or sales literature without prior written consent in
-;each case.
-
-;;(declare (unit factor)
-;;	 (uses lolevel)
-;;	 (export factor))
-
-;;; prime:products are products of small primes.
-;;; was (comlist:notevery (lambda (prd) (= 1 (gcd n prd))) comps))
-(declare
-  (hide primes-gcd? prime:prime-sqr prime:products prime:sieve prime:f prime:fo prime:fe) )
-
-(define (primes-gcd? n comps)
-  (not (let mapf ((lst comps))
-	 (or (null? lst) (and (= 1 (gcd n (car lst))) (mapf (cdr lst)))))))
-(define prime:prime-sqr 121)
-(define prime:products '(105))
-(define prime:sieve (vector 0 0 1 1 0 1 0 1 0 0 0))
-(letrec ((lp (lambda (comp comps primes nexp)
-	       (cond ((< comp (quotient (##sys#fudge 21) nexp))
-		      (let ((ncomp (* nexp comp)))
-			(lp ncomp comps
-			    (cons nexp primes)
-			    (next-prime nexp (cons ncomp comps)))))
-		     ((< (quotient comp nexp) (* nexp nexp))
-		      (set! prime:prime-sqr (* nexp nexp))
-		      (set! prime:sieve (make-vector nexp 0))
-		      (for-each (lambda (prime)
-				  (vector-set! prime:sieve prime 1))
-				primes)
-		      (set! prime:products (reverse (cons comp comps))))
-		     (else
-		      (lp nexp (cons comp comps)
-			  (cons nexp primes)
-			  (next-prime nexp (cons comp comps)))))))
-	 (next-prime (lambda (nexp comps)
-		       (set! comps (reverse comps))
-		       (do ((nexp (+ 2 nexp) (+ 2 nexp)))
-			   ((not (primes-gcd? nexp comps)) nexp)))))
-  (lp 3 '() '(2 3) 5))
-;;;;Lankinen's recursive factoring algorithm:
-;From: ld231782@longs.LANCE.ColoState.EDU (L. Detweiler)
-
-;                  |  undefined if n<0,
-;                  |  (u,v) if n=0,
-;Let f(u,v,b,n) := | [otherwise]
-;                  |  f(u+b,v,2b,(n-v)/2) or f(u,v+b,2b,(n-u)/2) if n odd
-;                  |  f(u,v,2b,n/2) or f(u+b,v+b,2b,(n-u-v-b)/2) if n even
-
-;Thm: f(1,1,2,(m-1)/2) = (p,q) iff pq=m for odd m.
-
-;It may be illuminating to consider the relation of the Lankinen function in
-;a `computational hierarchy' of other factoring functions.*  Assumptions are
-;made herein on the basis of conventional digital (binary) computers.  Also,
-;complexity orders are given for the worst case scenarios (when the number to
-;be factored is prime).  However, all algorithms would probably perform to
-;the same constant multiple of the given orders for complete composite
-;factorizations.
-
-;Thm: Eratosthenes' Sieve is very roughtly O(ln(n)/n) in time and
-;     O(n*log2(n)) in space.
-;Pf: It works with all prime factors less than n (about ln(n)/n by the prime
-;    number thm), requiring an array of size proportional to n with log2(n)
-;    space for each entry.
-
-;Thm: `Odd factors' is O((sqrt(n)/2)*log2(n)) in time and O(log2(n)) in
-;     space.
-;Pf: It tests all odd factors less than the square root of n (about
-;    sqrt(n)/2), with log2(n) time for each division.  It requires only
-;    log2(n) space for the number and divisors.
-
-;Thm: Lankinen's algorithm is O(sqrt(n)/2) in time and O((sqrt(n)/2)*log2(n))
-;     in space.
-;Pf: The algorithm is easily modified to seach only for factors p<q for all
-;    pq=m.  Then the recursive call tree forms a geometric progression
-;    starting at one, and doubling until reaching sqrt(n)/2, or a length of
-;    log2(sqrt(n)/2).  From the formula for a geometric progression, there is
-;    a total of about 2^log2(sqrt(n)/2) = sqrt(n)/2 calls.  Assuming that
-;    addition, subtraction, comparison, and multiplication/division by two
-;    occur in constant time, this implies O(sqrt(n)/2) time and a
-;    O((sqrt(n)/2)*log2(n)) requirement of stack space.
-
-(define (prime:f u v b n)
-  (if (<= n 0)
-      (cond ((negative? n) #f)
-	    ((= u 1) #f)
-	    ((= v 1) #f)
-	    ; Do both of these factors need to be factored?
-	    (else (append (or (prime:f 1 1 2 (quotient (- u 1) 2))
-			      (list u))
-			  (or (prime:f 1 1 2 (quotient (- v 1) 2))
-			      (list v)))))
-      (if (even? n)
-	  (or (prime:f u v (+ b b) (quotient n 2))
-	      (prime:f (+ u b) (+ v b) (+ b b) (quotient (- n (+ u v b)) 2)))
-	  (or (prime:f (+ u b) v (+ b b) (quotient (- n v) 2))
-	      (prime:f u (+ v b) (+ b b) (quotient (- n u) 2))))))
-
-(define (prime:fo m)
-  (let* ((s (gcd m (car prime:products)))
-	 (r (quotient m s)))
-    (if (= 1 s)
-	(or (prime:f 1 1 2 (quotient (- m 1) 2)) (list m))
-	(append
-	 (if (= 1 r) '()
-	     (or (prime:f 1 1 2 (quotient (- r 1) 2)) (list r)))
-	 (or (prime:f 1 1 2 (quotient (- s 1) 2)) (list s))))))
-
-(define (prime:fe m)
-  (if (even? m)
-      (cons 2 (prime:fe (quotient m 2)))
-      (if (eqv? 1 m)
-	  '()
-	  (prime:fo m))))
-
-;;@body
-;;Returns a list of the prime factors of @1.  The order of the
-;;factors is unspecified.  In order to obtain a sorted list do
-;;@code{(sort! (factor @var{k}) <)}.
-(define (factor k)
-  (case k
-    ((-1 0 1) (list k))
-    (else (if (negative? k)
-	      (cons -1 (prime:fe (- k)))
-	      (prime:fe k)))))
