@@ -1,6 +1,6 @@
 ;;;; csi.scm - Simple interpreter stub for CHICKEN
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -33,29 +33,26 @@
 ; Germany
 
 
-(declare (uses match-support))
+(declare (uses match))
 
 (declare
   (usual-integrations)
   (disable-interrupts)
+  (disable-warning var)
   (run-time-macros)
   (foreign-declare #<<EOF
-#if defined(_MSC_VER) && defined(_WIN32)
+#if (defined(_MSC_VER) && defined(_WIN32)) || defined(HAVE_DIRECT_H)
 # include <direct.h>
 #else
 # define _getcwd(buf, len)       NULL
 #endif
-
-#ifdef __DJGPP__
-# include <unistd.h>
-#endif
 EOF
 ) )
 
 (include "chicken-more-macros")
-(include "chicken-match-macros")
 (include "banner")
 
+
 #{csi 
   print-usage print-banner
   run hexdump del 
@@ -63,11 +60,12 @@ EOF
   report describe dump hexdump bytevector-data get-config
   deldups tty-input?
   history-list history-count history-add history-ref
-  trace-indent trace-indent-level traced-procedures traced-procedure-entry traced-procedure-exit}
+  trace-indent trace-indent-level traced-procedure-entry traced-procedure-exit}
 
 (declare
   (hide parse-option-string bytevector-data banner member* canonicalize-args do-trace do-untrace
-	findall trace-indent command-table) )
+	traced-procedures describer-table
+	findall trace-indent command-table do-break do-unbreak broken-procedures) )
 
 
 ;;; Parameters:
@@ -88,6 +86,7 @@ EOF
 
     -h  -help  --help           display this text and exit
     -v  -version                display version and exit
+        -release                print release number and exit
     -i  -case-insensitive       enable case-insensitive reading
     -e  -eval EXPRESSION        evaluate given expression
     -D  -feature SYMBOL         register feature identifier
@@ -107,7 +106,7 @@ EOF
 ") )
 
 (define (print-banner)
-  (print banner (chicken-version #t) "\n(c)2000-2005 Felix L. Winkelmann") )
+  (print banner (chicken-version #t) "\n" copyright) )
 
 
 ;;; Reader for REPL history:
@@ -225,6 +224,7 @@ EOF
   (let ((eval eval)
 	(load-noisily load-noisily)
 	(read read)
+	(singlestep singlestep)
 	(hash-table-ref hash-table-ref)
 	(hash-table-walk hash-table-walk)
 	(read-line read-line)
@@ -286,8 +286,33 @@ EOF
 			   (receive rs (time (eval x))
 			     (history-add rs)
 			     (apply values rs) ) ) )
-			((tr) (do-trace (string-split (read-line))))
-			((utr) (do-untrace (string-split (read-line))))
+			((tr) (do-trace (map string->symbol (string-split (read-line)))))
+			((utr) (do-untrace (map string->symbol (string-split (read-line)))))
+			((br) (do-break (map string->symbol (string-split (read-line)))))
+			((ubr) (do-unbreak (map string->symbol (string-split (read-line)))))
+			((breakall) 
+			 (set! ##sys#break-in-thread #t) ) 
+			((breakonly)
+			 (set! ##sys#break-in-thread (eval (read))) )
+			((info)
+			 (when (pair? traced-procedures)
+			   (printf "Traced: ~s~%" (map car traced-procedures)) )
+			 (when (pair? broken-procedures)
+			   (printf "Breakpoints: ~s~%" (map car broken-procedures)) ) )
+			((c)
+			 (cond (##sys#last-breakpoint
+				(let ((exn ##sys#last-breakpoint))
+				  (set! ##sys#last-breakpoint #f)
+				  (##sys#break-resume exn) ) )
+			       (else (display "no breakpoint pending\n") ) ) )
+			((exn) 
+			 (when ##sys#last-exception
+			   (history-add (list ##sys#last-exception))
+			   (describe ##sys#last-exception) ) )
+			((step)
+			 (let ((x (read)))
+			   (read-line)
+			   (singlestep (eval `(lambda () ,x))) ) )
 			((s)
 			 (let* ((str (read-line))
 				(r (system str)) )
@@ -309,6 +334,14 @@ EOF
  ,s TEXT ...       Execute shell-command
  ,tr NAME ...      Trace procedures
  ,utr NAME ...     Untrace procedures
+ ,br NAME ...      Set breakpoints
+ ,ubr NAME ...     Remove breakpoints
+ ,breakall         Break in all threads
+ ,breakonly THREAD Break only in specified thread
+ ,c                Continue from breakpoint
+ ,info             List traced procedures and breakpoints
+ ,step EXPR        Execute EXPR in single-stepping mode
+ ,exn              Describe last exception
  ,t EXP            Evaluate form and print elapsed time
  ,x EXP            Pretty print macroexpanded expression EXP\n")
 			 (hash-table-walk
@@ -324,7 +357,7 @@ EOF
 			 (##sys#void) ) ) ) ) ) )
 	    (else
 	     (receive rs (eval form)
-	       (when (tty-input?) (history-add rs))
+	       (history-add rs)
 	       (apply values rs) ) ) ) ) ) )
 
 
@@ -341,6 +374,7 @@ EOF
 
 (define trace-indent-level 0)
 (define traced-procedures '())
+(define broken-procedures '())
 
 (define trace-indent
   (lambda ()
@@ -373,36 +407,76 @@ EOF
 
 (define do-trace
   (lambda (names)
-    (for-each
-     (lambda (s)
-       (let ((s (string->symbol s)))
-	 (if (assq s traced-procedures)
-	     (##sys#warn "procedure already traced" s)
-	     (let ((old (##sys#slot s 0)))
-	       (cond ((not (procedure? old)) (##sys#error "can not trace non-procedure" s))
-		     (else
-		      (set! traced-procedures (cons (cons s old) traced-procedures))
-		      (##sys#setslot
-		       s 0
-		       (lambda args
-			 (traced-procedure-entry s args)
-			 (call-with-values (lambda () (apply old args))
-			   (lambda results
-			     (traced-procedure-exit s results) 
-			     (apply values results) ) ) ) ) ) ) ) ) ) )
-     names) ) )
+    (if (null? names)
+	(for-each (lambda (a) (print (car a))) traced-procedures) 
+	(for-each
+	 (lambda (s)
+	   (let ((s (macroexpand s)))
+	     (cond ((assq s traced-procedures)
+		    (##sys#warn "procedure already traced" s) )
+		   ((assq s broken-procedures)
+		    (##sys#warn "procedure already has breakpoint") )
+		   (else
+		    (let ((old (##sys#slot s 0)))
+		      (cond ((not (procedure? old)) (##sys#error "can not trace non-procedure" s))
+			    (else
+			     (set! traced-procedures (cons (cons s old) traced-procedures))
+			     (##sys#setslot
+			      s 0
+			      (lambda args
+				(traced-procedure-entry s args)
+				(call-with-values (lambda () (apply old args))
+				  (lambda results
+				    (traced-procedure-exit s results) 
+				    (apply values results) ) ) ) ) ) ) ) ) ) ) )
+	 names) ) ) )
 
 (define do-untrace 
   (lambda (names)
     (for-each
      (lambda (s)
-       (let* ((s (string->symbol s))
+       (let* ((s (macroexpand s))
 	      (p (assq s traced-procedures)) )
 	 (cond ((not p) (##sys#warn "procedure not traced" s))
 	       (else
 		(##sys#setslot s 0 (cdr p))
-		(set! traced-procedures (del s traced-procedures eq?) ) ) ) ) )
-     (if (null? names) (map (lambda (x) (symbol->string (car x))) traced-procedures) names) ) ) )
+		(set! traced-procedures (del p traced-procedures eq?) ) ) ) ) )
+     names) ) )
+
+(define do-break
+  (lambda (names)
+    (if (null? names)
+	(for-each (lambda (b) (print (car a))) broken-procedures) 
+	(for-each
+	 (lambda (s)
+	   (let* ((s (macroexpand s))
+		  (a (assq s traced-procedures)))
+	     (when a
+	       (##sys#warn "un-tracing procedure" s)
+	       (##sys#setslot s 0 (cdr a))
+	       (set! traced-procedures (del a traced-procedures eq?)) )
+	     (let ((old (##sys#slot s 0)))
+	       (cond ((not (procedure? old)) (##sys#error "can not set breakpoint on non-procedure" s))
+		     (else
+		      (set! broken-procedures (cons (cons s old) broken-procedures))
+		      (##sys#setslot
+		       s 0
+		       (lambda args
+			 (##sys#break-entry s args)
+			 (##sys#apply old args) ) ) ) ) ) ) )
+	 names) ) ) )
+
+(define do-unbreak
+  (lambda (names)
+    (for-each
+     (lambda (s)
+       (let* ((s (macroexpand s))
+	      (p (assq s broken-procedures)) )
+	 (cond ((not p) (##sys#warn "procedure has no breakpoint" s))
+	       (else
+		(##sys#setslot s 0 (cdr p))
+		(set! broken-procedures (del p broken-procedures eq?) ) ) ) ) )
+     names) ) )
 
 
 ;;; Parse options from string:
@@ -425,6 +499,7 @@ EOF
 
 (define report
   (let ([printf printf]
+	(chop chop)
 	[with-output-to-port with-output-to-port] 
 	[current-output-port current-output-port] )
     (lambda port
@@ -434,10 +509,12 @@ EOF
 	  (let ([sinfo (##sys#symbol-table-info)]
 		[minfo (memory-statistics)] )
 	    (define (shorten n) (/ (truncate (* n 100)) 100))
-	    (printf "Features:~%")
+	    (printf "Features:")
 	    (for-each
-	     (cut print #\tab <>) 
-	     (map keyword->string ##sys#features) )
+	     (lambda (lst) 
+	       (newline)
+	       (for-each (cut print* #\tab <>) lst) )
+	     (chop (map keyword->string ##sys#features) 8) )
 	    (printf "~%~
                    Machine type:    \t~A ~A~%~
                    Software type:   \t~A~%~
@@ -446,7 +523,7 @@ EOF
                    Include path:    \t~A~%~
                    Symbol-table load:\t~S~%  ~
                      Avg bucket length:\t~S~%  ~
-                     Total symbols:\t~S~%~%~
+                     Total symbols:\t~S~%~
                    Memory:\theap size is ~S bytes~A with ~S bytes currently in use~%~  
                      nursery size is ~S bytes, stack grows ~A~%"
 		    (machine-type)
@@ -483,6 +560,8 @@ EOF
 
 (define-constant max-describe-lines 40)
 
+(define describer-table (make-hash-table eq?))
+
 (define describe
   (let ([sprintf sprintf]
 	[printf printf] 
@@ -490,6 +569,7 @@ EOF
 	[length length]
 	[list-ref list-ref]
 	[string-ref string-ref]
+	(hash-table-ref/default hash-table-ref/default)
 	[hash-table-walk hash-table-walk] )
     (lambda (x . port)
       (let ([out (:optional port ##sys#standard-output)])
@@ -537,7 +617,7 @@ EOF
 	       (let ([len (##sys#size x)])
 		 (if (and (> len 3)
 			  (memq #:tinyclos ##sys#features)
-			  (eq? ##tinyclos#entity-tag (##sys#slot x (fx- len 1))) )
+			  (eq? ##tinyclos#entity-tag (##sys#slot x (fx- len 1))) ) ;XXX handle this in tinyclos egg...
 		     (describe-object x out)
 		     (descseq 
 		      (sprintf "procedure with code pointer ~X" (##sys#peek-unsigned-integer x 0))
@@ -581,14 +661,10 @@ EOF
 	       (hash-table-walk
 		x
 		(lambda (k v) (fprintf out " ~S\t-> ~S~%" k v)) ) ]
-	      [(##sys#structure? x 'environment)
-	       (if (##sys#slot x 1)
-		   (fprintf out "environment~%") 
-		   (fprintf out "the interaction environment~%") ) ]
 	      [(##sys#structure? x 'array)
 	       (let* ([shp (##sys#slot x 3)]
 		      [vec (##sys#slot x 1)] 
-		      (n (##sys#siize vec)) )
+		      (n (##sys#size vec)) )
 		 (fprintf out "array of rank ~S with ~A element~a~% shape: "
 			  (fx/ (##sys#size shp) 2)
 			  n
@@ -599,19 +675,36 @@ EOF
 		       (fprintf out " ~A..~A " (##sys#slot s 0) (##sys#slot next 0))
 		       (loop (##sys#slot next 1)) ) ) )
 		 (##sys#write-char-0 #\newline out) ) ]
+	      [(##sys#structure? x 'condition)
+	       (fprintf out "condition: ~s~%" (##sys#slot x 1))
+	       (for-each
+		(lambda (k)
+		  (fprintf out " ~s~%" k)
+		  (let loop ((props (##sys#slot x 2)))
+		    (unless (null? props)
+		      (when (eq? k (caar props))
+			(fprintf out "\t~s: ~s~%" (cdar props) (cadr props)) )
+		      (loop (cddr props)) ) ) )
+		(##sys#slot x 1) ) ]
 	      [(and (##sys#structure? x 'meroon-instance) (provided? 'meroon))
 	       (unveil x out) ]
 	      [(##sys#generic-structure? x)
 	       (let ([st (##sys#slot x 0)])
-		 (let ([data (assq st bytevector-data)] )
-		   (if data
-		       (apply descseq (append (map eval (cdr data)) (list 0)))
-		       (begin
-			 (fprintf out "structure of type `~S':~%" (##sys#slot x 0))
-			 (descseq #f ##sys#size ##sys#slot 1) ) ) ) ) ]
+		 (cond ((hash-table-ref/default describer-table st #f) => (cut <> x out))
+		       ((assq st bytevector-data) =>
+			(lambda (data)
+			  (apply descseq (append (map eval (cdr data)) (list 0)))) )
+		       (else
+			(fprintf out "structure of type `~S':~%" (##sys#slot x 0))
+			(descseq #f ##sys#size ##sys#slot 1) ) ) ) ]
 	      [else (fprintf out "unknown object~%")] )
 	(##sys#void) ) ) ) )
 
+(define set-describer!
+  (let ((hash-table-set! hash-table-set!))
+    (lambda (tag proc)
+      (hash-table-set! describer-table tag proc) ) ) )
+
 
 ;;; Display hexdump:
 
@@ -738,11 +831,11 @@ EOF
 		 (else (find (cdr ks))) ) ) ) ) )
 
 (define-constant short-options 
-  '(#\k #\s #\v #\h #\D #\e #\i #\R #\b #\n #\q #\w #\- #\I) )
+  '(#\k #\s #\v #\h #\D #\e #\i #\R #\b #\n #\q #\w #\- #\I #f) )
 
 (define-constant long-options
   '("-keyword-style" "-script" "-version" "-help" "--help" "--" "-feature" "-eval" "-case-insensitive"
-    "-require-extension" "-batch" "-quiet" "-no-warnings" "-no-init" "-include-path") )
+    "-require-extension" "-batch" "-quiet" "-no-warnings" "-no-init" "-include-path" "-release") )
 
 (define (canonicalize-args args)
   (let loop ((args args))
@@ -815,6 +908,9 @@ EOF
       (when (member* '("-v" "-version") args)
 	(print-banner)
 	(exit 0) )
+      (when (member "-release" args)
+	(print (chicken-version))
+	(exit 0) )
       (when (member* '("-w" "-no-warnings") args)
 	(unless quiet (display "Warnings are disabled\n"))
 	(set! ##sys#warnings-enabled #f) )
