@@ -1,6 +1,6 @@
 ;;;; eval.scm - Interpreter for CHICKEN
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -35,38 +35,59 @@
 
 (declare
   (unit eval)
-  (hide ##sys#unregister-macro ##sys#macroexpand-0 ##sys#split-at-separator
+  (disable-warning var)
+  (hide ##sys#unregister-macro ##sys#split-at-separator
 	##sys#r4rs-environment ##sys#r5rs-environment 
-	##sys#interaction-environment pds pdss pxss)
-  (foreign-declare #<<EOF
+	##sys#interaction-environment pds pdss pxss) )
 
-#ifndef C_INSTALL_LIB_HOME
-# define C_INSTALL_LIB_HOME    "."
+#>
+#ifndef C_INSTALL_EGG_HOME
+# define C_INSTALL_EGG_HOME    "."
 #endif
 
-EOF
-) )
+#ifndef C_INSTALL_SHARE_HOME
+# define C_INSTALL_SHARE_HOME NULL
+#endif
+<#
 
 (cond-expand
  [paranoia]
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure 
      ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string ##sys#load-library
      ##sys#for-each ##sys#map ##sys#setslot ##sys#allocate-vector ##sys#check-pair ##sys#not-a-proper-list-error
      ##sys#check-symbol ##sys#check-vector ##sys#floor ##sys#ceiling ##sys#truncate ##sys#round 
      ##sys#check-number ##sys#cons-flonum ##sys#copy-env-table
      ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg ##sys#print ##sys#check-structure 
-     ##sys#make-structure ##sys#test-feature ##sys#interpreter-toplevel-macroexpand-hook
+     ##sys#make-structure ##sys#feature? ##sys#interpreter-toplevel-macroexpand-hook
      ##sys#error-handler ##sys#hash-symbol ##sys#register-macro ##sys#check-syntax
      ##sys#hash-table-ref ##sys#hash-table-set! ##sys#canonicalize-body ##sys#decompose-lambda-list
      ##sys#make-c-string ##sys#resolve-include-filename ##sys#register-macro-2 
-     ##sys#load ##sys#error ##sys#warn ##sys#hash-table-location
+     ##sys#load ##sys#error ##sys#warn ##sys#hash-table-location ##sys#expand-home-path
      ##sys#make-flonum ##sys#make-pointer ##sys#null-pointer ##sys#address->pointer 
-     ##sys#pointer->address ##sys#compile-to-closure
+     ##sys#pointer->address ##sys#compile-to-closure ##sys#make-string ##sys#make-lambda-info
+     ##sys#number? ##sys#symbol->qualified-string ##sys#decorate-lambda ##sys#string-append
      ##sys#ensure-heap-reserve ##sys#syntax-error-hook ##sys#read-prompt-hook
      ##sys#repl-eval-hook ##sys#append ##sys#eval-decorator
+     open-output-string get-output-string make-parameter software-type software-version machine-type
+     build-platform getenv set-extensions-specifier! ##sys#string->symbol list->vector
+     extension-information syntax-error ->string chicken-home ##sys#expand-curried-define
+     ##sys#match-expression vector->list store-string open-input-string eval ##sys#gc
+     with-exception-handler print-error-message read-char read ##sys#read-error
+     ##sys#reset-handler call-with-current-continuation ##sys#peek-char-0 ##sys#read-char-0
+     ##sys#clear-trace-buffer ##sys#write-char-0 print-call-chain ##sys#with-print-length-limit
+     repl-prompt ##sys#flush-output ##sys#extended-lambda-list? keyword? get-line-number
+     symbol->string string-append display ##sys#repository-path ##sys#file-info make-vector
+     ##sys#make-vector string-copy vector->list ##sys#do-the-right-thing ##sys#->feature-id
+     ##sys#extension-information ##sys#symbol->string ##sys#canonicalize-extension-path
+     file-exists? ##sys#load-extension ##sys#find-extension ##sys#substring reverse
+     dynamic-load-libraries ##sys#string->c-identifier load-verbose ##sys#load ##sys#get-keyword
+     port? ##sys#file-info ##sys#signal-hook ##sys#dload open-input-file close-input-port
+     read write newline ##sys#eval-handler ##sys#set-dlopen-flags! cadadr ##sys#lookup-runtime-requirements
+     map string->keyword ##sys#abort
      ##sys#macroexpand-0 ##sys#macroexpand-1-local) ) ] )
 
 (cond-expand
@@ -83,16 +104,54 @@ EOF
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "eval.exports"))])
 
 
-(include "parameters")
-
-(define-foreign-variable install-lib-home c-string "C_INSTALL_LIB_HOME")
+(define-foreign-variable install-egg-home c-string "C_INSTALL_EGG_HOME")
+(define-foreign-variable installation-home c-string "C_INSTALL_SHARE_HOME")
 
 (define pds ##sys#pathname-directory-separator)
 (define pdss (string ##sys#pathname-directory-separator))
 
+(define ##sys#core-library-modules
+  '(extras lolevel utils tcp regex posix match srfi-1 srfi-4 srfi-14 srfi-18 srfi-13))
+
+(define-constant macro-table-size 301)
+(define-constant default-dynamic-load-libraries '("libchicken"))
+(define-constant cygwin-default-dynamic-load-libraries '("cygchicken-0"))
+(define-constant macosx-load-library-extension ".dylib")
+(define-constant windows-load-library-extension ".dll")
+(define-constant hppa-load-library-extension ".sl")
+(define-constant default-load-library-extension ".so")
+(define-constant environment-table-size 301)
+(define-constant source-file-extension ".scm")
+(define-constant setup-file-extension "setup-info")
+(define-constant repository-environment-variable "CHICKEN_REPOSITORY")
+(define-constant special-syntax-files '(chicken-ffi-macros chicken-more-macros))
+
+(define-constant builtin-features
+  '(chicken srfi-23 srfi-30 srfi-39 srfi-6 srfi-10 srfi-2 srfi-31
+	    srfi-69 srfi-28) )		; these are actually in extras, but that is used by default
+
+(define-constant builtin-features/compiled
+  '(srfi-11 srfi-8 srfi-6 srfi-16 srfi-15 srfi-26 srfi-55 srfi-9 srfi-17) )
+
+
+;;; System settings
+
+(define chicken-home
+  (let ([getenv getenv])
+    (lambda ()
+      (or (getenv "CHICKEN_HOME")
+	  (and-let* ((p (getenv "CHICKEN_PREFIX")))
+	    (##sys#string-append 
+	     p
+	     (if (char=? (string-ref p (fx- (##sys#size p) 1)) ##sys#pathname-directory-separator)
+		 "share"
+		 "/share") ) )
+	  installation-home) ) ) )
+
 
 ;;; Macro handling:
 
@@ -143,11 +202,11 @@ EOF
 			'()
 			(let ([p (car ps)]
 			      [r (cdr ps)])
-			  (if (and (eq? 'message p)
+			  (if (and (equal? '(exn . message) p)
 				   (pair? r)
 				   (string? (car r)) )
 			      (cons 
-			       'message
+			       '(exn . message)
 			       (cons (string-append
 				      "during expansion of (" (##sys#slot name 1) " ...) - "
 				      (car r) )
@@ -208,6 +267,11 @@ EOF
 (define (##sys#macroexpand-1-local exp me) (##sys#macroexpand-0 exp me))
 
 
+;;; For the compiler
+
+(define ##sys#enable-runtime-macros #f)
+
+
 ;;; User-level macroexpansion
 
 (define (macroexpand exp . me)
@@ -222,27 +286,7 @@ EOF
   (##sys#macroexpand-0 exp (if (pair? me) (car me) '())) )
 
 
-;;; Misc 
-
-(define ##sys#enable-runtime-macros #f)
-
-(define (##sys#undefine-non-standard-macros leave)
-  (let ([leave (##sys#append leave '(define and or cond case let* letrec do quasiquote delay))])
-    (do ([i 0 (fx+ i 1)])
-	((fx>= i macro-table-size))
-      (##sys#setslot 
-       ##sys#macro-environment i
-       (let loop ([bs (##sys#slot ##sys#macro-environment i)])
-	 (if (null? bs)
-	     '()
-	     (let ([b (##sys#slot bs 0)]
-		   [r (##sys#slot bs 1)] )
-	       (if (memq (##sys#slot b 0) leave)
-		   (cons b (loop r))
-		   (loop r) ) ) ) ) ) ) ) )
-
-
-;;; Extended (DSSSL-style) lambda lists:
+;;; Extended (DSSSL-style) lambda lists
 ;
 ; Assumptions:
 ;
@@ -1022,6 +1066,7 @@ EOF
 
 (define (##sys#abort-load) #f)
 (define ##sys#current-load-file #f)
+(define ##sys#current-load-path "")
 
 (define-foreign-variable _dlerror c-string "C_dlerror")
 
@@ -1044,6 +1089,7 @@ EOF
       [write write]
       [display display]
       [newline newline]
+      [eval eval]
       [open-input-file open-input-file]
       [close-input-port close-input-port]
       [string-append string-append] 
@@ -1052,7 +1098,8 @@ EOF
   (define (has-sep? str)
     (let loop ([i (fx- (##sys#size str) 1)])
       (and (not (zero? i))
-	   (or (char=? pds (##core#inline "C_subchar" str i))
+	   (if (char=? pds (##core#inline "C_subchar" str i))
+	       i
 	       (loop (fx- i 1)) ) ) ) )
   (define (badfile x)
     (##sys#signal-hook #:type-error 'load "bad argument type - not a port or string" x) )
@@ -1077,11 +1124,7 @@ EOF
 			    (if (##sys#file-info fname3)
 				fname3
 				(and (not isdir) input) ) ) ) ) ] ) ]
-	    [evproc (or evaluator 
-			(lambda (x . env)
-			  (apply (##sys#eval-handler)
-				 (##sys#interpreter-toplevel-macroexpand-hook x)
-				 env) ) ) ] )
+	    [evproc (or evaluator eval)] )
 	(cond [(and (string? input) (not fname))
 	       (##sys#signal-hook #:file-error 'load "can not open file" input) ]
 	      [(and (load-verbose) fname)
@@ -1096,6 +1139,9 @@ EOF
 	     (lambda (abrt)
 	       (fluid-let ([##sys#read-error-with-line-number #t]
 			   [##sys#current-load-file fname]
+			   [##sys#current-load-path
+			    (let ((i (has-sep? fname)))
+			      (if i (##sys#substring fname 0 (fx+ i 1)) "") ) ]
 			   [##sys#abort-load (lambda () (abrt #f))] )
 		 (let ([in (if fname (open-input-file fname) input)])
 		   (##sys#dynamic-wind
@@ -1104,27 +1150,36 @@ EOF
 		      (let ([c1 (peek-char in)])
 			(when (char=? c1 (integer->char 127))
 			  (##sys#error 'load "unable to load compiled module" fname _dlerror) ) )
-		      (do ((x (read in) (read in)))
-			  ((eof-object? x))
-			(when printer (printer x))
-			(##sys#call-with-values
-			 (lambda () 
-			   (if timer
-			       (time (evproc x)) 
-			       (evproc x) ) )
-			 (lambda results
-			   (when pf
-			     (for-each
-			      (lambda (r) 
-				(write r)
-				(newline) )
-			      results) ) ) ) ) )
+		      (let ((x1 (read in)))
+			(do ((x x1 (read in)))
+			    ((eof-object? x))
+			  (when printer (printer x))
+			  (##sys#call-with-values
+			   (lambda () 
+			     (if timer
+				 (time (evproc x)) 
+				 (evproc x) ) )
+			   (lambda results
+			     (when pf
+			       (for-each
+				(lambda (r) 
+				  (write r)
+				  (newline) )
+				results) ) ) ) ) ) )
 		    (lambda () (close-input-port in)) ) ) ) ) ) )
 	(##core#undefined) ) ) )
   (set! load
     (lambda (filename . evaluator)
       (fluid-let ([##sys#current-namespace ##sys#current-namespace])
 	(##sys#load filename (:optional evaluator #f) #f) ) ) )
+  (set! load-relative
+    (lambda (filename . evaluator)
+      (fluid-let ([##sys#current-namespace ##sys#current-namespace])
+	(##sys#load
+	 (if (char=? pds (string-ref filename 0))
+	     filename
+	     (##sys#string-append ##sys#current-load-path filename) )
+	 (:optional evaluator #f) #f) ) ) )
   (set! load-noisily
     (lambda (filename #!key (evaluator #f) (time #f) (printer #f))
       (fluid-let ([##sys#current-namespace ##sys#current-namespace])
@@ -1143,14 +1198,10 @@ EOF
 	      (eq? (machine-type) 'hppa)) default-load-library-extension]
 	[else ##sys#load-library-extension] ) )
 
-(define ##sys#default-dynamic-load-libraries
-  (case (software-type)
-    [(windows)
-     (case (build-platform)
-       [(msvc) msvc-default-dynamic-load-libraries]
-       [(mingw32) mingw-default-dynamic-load-libraries]
-       [else cygwin-default-dynamic-load-libraries] ) ]
-    [else unix-default-dynamic-load-libraries] ) )
+(define ##sys#default-dynamic-load-libraries 
+  (case (build-platform)
+    ((cygwin) cygwin-default-dynamic-load-libraries)
+    (else default-dynamic-load-libraries) ) )
 
 (define dynamic-load-libraries 
   (make-parameter
@@ -1241,8 +1292,8 @@ EOF
 (define ##sys#repository-path
   (make-parameter 
    (or (getenv repository-environment-variable)
-       (getenv home-environment-variable)
-       install-lib-home) ) )
+       (getenv "CHICKEN_HOME")
+       install-egg-home) ) )
 
 (define repository-path ##sys#repository-path)
 
@@ -1271,7 +1322,7 @@ EOF
 	    (else (##sys#check-symbol id loc)) )
       (let ([p (##sys#canonicalize-extension-path id loc)])
 	(cond ((member p ##sys#loaded-extensions))
-	      ((memq id core-library-modules)
+	      ((memq id ##sys#core-library-modules)
 	       (##sys#load-library id #f) )
 	      (else
 	       (let ([id2 (##sys#find-extension p #t)])
@@ -1316,8 +1367,6 @@ EOF
 	     (rpath (string-append (##sys#repository-path) pdss p ".")) )
 	(cond ((file-exists? (string-append rpath setup-file-extension))
 	       => (cut with-input-from-file <> read) )
-	      ((file-exists? (string-append rpath alternative-setup-file-extension))
-	       => (cut with-input-from-file <> read) )
 	      (else #f) ) ) ) ) )
 
 (define (extension-information ext)
@@ -1340,8 +1389,7 @@ EOF
 	     (loop1 (cdr ids)) ) ) ) ) ) )
 
 (define ##sys#do-the-right-thing
-  (let ([vector->list vector->list]
-	[test-feature? test-feature?] )
+  (let ([vector->list vector->list])
     (lambda (id comp?)
       (define (add-req id)
 	(when comp?
@@ -1354,7 +1402,7 @@ EOF
 	(cond [(or (memq id builtin-features)
 		   (if comp?
 		       (memq id builtin-features/compiled)
-		       (test-feature? id) ) )
+		       (##sys#feature? id) ) )
 	       (values '(##sys#void) #t) ]
 	      [(memq id special-syntax-files)
 	       (let ([fid (##sys#->feature-id id)])
@@ -1362,16 +1410,12 @@ EOF
 		   (##sys#load (##sys#resolve-include-filename (##sys#symbol->string id) #t) #f #f) 
 		   (set! ##sys#features (cons fid ##sys#features)) )
 		 (values '(##sys#void) #t) ) ]
-	      [(memq id core-library-modules)
-	       (let* ([a (assq id specially-handled-syntax-files)]
-		      [id (if a (cadr a) id)] )
-		 (when (and a (not (memq (##sys#->feature-id (car a)) ##sys#features)))
-		   (##sys#load (##sys#resolve-include-filename (caddr a) #t) #f #f))
-		 (values
-		  (if comp?
-		      `(##core#declare '(uses ,id))
-		      `(load-library ',id) )
-		  #t) ) ]
+	      [(memq id ##sys#core-library-modules)
+	       (values
+		(if comp?
+		    `(##core#declare '(uses ,id))
+		    `(load-library ',id) )
+		#t) ]
 	      [else
 	       (let ([info (##sys#extension-information id 'require-extension)])
 		 (cond [info
@@ -1440,6 +1484,11 @@ EOF
 (set-extension-specifier!
  'version
  (lambda (spec _)
+   (define (->string x)
+     (cond ((string? x) x)
+	   ((symbol? x) (##sys#slot x 1))
+	   ((number? x) (##sys#number->string x))
+	   (else (error "invalid extension version" x)) ) )
    (match spec
      (('version id v)
       (let* ((info (extension-information id))
@@ -1533,7 +1582,7 @@ EOF
      vector-length vector->list list->vector vector-fill! procedure? map for-each apply force 
      call-with-current-continuation input-port? output-port? current-input-port current-output-port
      call-with-input-file call-with-output-file open-input-file open-output-file close-input-port
-     close-output-port load transcript-on transcript-off read eof-object? read-char peek-char
+     close-output-port load read eof-object? read-char peek-char
      write display write-char newline with-input-from-file with-output-to-file ##sys#call-with-values
      ##sys#values ##sys#dynamic-wind ##sys#void
      ##sys#list->vector ##sys#list ##sys#append ##sys#cons ##sys#make-promise) )
@@ -1550,8 +1599,7 @@ EOF
     (if h (list h) '())) )
 
 (define ##sys#resolve-include-filename
-  (let ((string string)
-	(string-append string-append) )
+  (let ((string-append string-append) )
     (define (exists? fname)
       (let ([info (##sys#file-info fname)])
 	(and info (not (eq? 1 (##sys#slot info 4)))) ) )
@@ -1764,12 +1812,20 @@ EOF
 	     (##sys#check-syntax 'cond clause '#(_ 1))
 	     (cond ((eq? 'else (car clause)) `(begin ,@(cdr clause)))
 		   ((eq? (cdr clause) '()) `(or ,(car clause) ,(expand rclauses)))
-		   ((eq? '=> (car (cdr clause)))
+		   ((eq? '=> (cadr clause))
 		    (let ((tmp (gensym)))
 		      `(let ((,tmp ,(car clause)))
 			 (if ,tmp
-			     (,(car (cdr (cdr clause))) ,tmp)
+			     (,(caddr clause) ,tmp)
 			     ,(expand rclauses) ) ) ) )
+		   ((and (list? clause) (fx= (length clause) 4) (eq? '=> (caddr clause)))
+		    (let ((tmp (gensym)))
+		      `(##sys#call-with-values
+			(lambda () ,(car clause))
+			(lambda ,tmp
+			  (if (##sys#apply ,(cadr clause) ,tmp)
+			      (##sys#apply ,(cadddr clause) ,tmp)
+			      ,(expand rclauses) ) ) ) ) )
 		   (else `(if ,(car clause) 
 			      (begin ,@(cdr clause))
 			      ,(expand rclauses) ) ) ) ) ) ) ) ) )
@@ -1912,13 +1968,9 @@ EOF
 ;;; SRFI-0 support code:
 
 (set! ##sys#features
-  (append '(#:srfi-8 #:srfi-6 #:srfi-2 #:srfi-0 #:srfi-10 #:srfi-9 #:srfi-55) 
+  (append '(#:srfi-8 #:srfi-6 #:srfi-2 #:srfi-0 #:srfi-10 #:srfi-9 #:srfi-55 #:srfi-61) 
 	  ##sys#features))
 
-(define (##sys#test-feature f)
-  (let ([f (##sys#->feature-id f)])
-    (memq f ##sys#features) ) )
-
 
 ;;;; Read-Eval-Print loop:
 
@@ -1929,10 +1981,9 @@ EOF
 (define repl-prompt (make-parameter (lambda () "#;> ")))
 
 (define ##sys#read-prompt-hook
-  (let ([display display]
-	[repl-prompt repl-prompt] )
+  (let ([repl-prompt repl-prompt])
     (lambda () 
-      (display ((repl-prompt)))
+      (##sys#print ((repl-prompt)) #f ##sys#standard-output)
       (##sys#flush-output ##sys#standard-output) ) ) )
 
 (define ##sys#clear-trace-buffer (foreign-lambda void "C_clear_trace_buffer"))
@@ -1940,23 +1991,22 @@ EOF
 (define repl
   (let ([eval eval]
 	[read read]
-	[write write]
 	[call-with-current-continuation call-with-current-continuation]
-	[display display]
 	[print-call-chain print-call-chain]
+	[flush-output flush-output]
 	[reset reset] )
     (lambda ()
 
-      (define (write-one x)
-	(##sys#with-print-length-limit ##sys#repl-print-length-limit (cut write x))
-	(##sys#write-char-0 #\newline ##sys#standard-output) )
+      (define (write-one x port)
+	(##sys#with-print-length-limit ##sys#repl-print-length-limit (cut ##sys#print x #t port))
+	(##sys#write-char-0 #\newline port) )
 
       (define (write-err xs)
-	(for-each write-one xs) )
+	(for-each (cut write-one <> ##sys#standard-error) xs) )
 
       (define (write-results xs)
 	(unless (or (null? xs) (eq? (##core#undefined) (car xs)))
-	  (for-each write-one xs) ) )
+	  (for-each (cut write-one <> ##sys#standard-output) xs) ) )
 
       (let ([stdin ##sys#standard-input]
 	    [stdout ##sys#standard-output]
@@ -1979,16 +2029,17 @@ EOF
 	   (##sys#error-handler
 	    (lambda (msg . args)
 	      (resetports)
-	      (display "Error: ")
-	      (display msg)
+	      (##sys#print "Error: " #f ##sys#standard-error)
+	      (##sys#print msg #f ##sys#standard-error)
 	      (if (and (pair? args) (null? (cdr args)))
 		  (begin
-		    (display ": ")
+		    (##sys#print ": " #f ##sys#standard-error)
 		    (write-err args) )
 		  (begin
-		    (##sys#write-char-0 #\newline ##sys#standard-output)
+		    (##sys#write-char-0 #\newline ##sys#standard-error)
 		    (write-err args) ) )
-	      (print-call-chain ##sys#standard-output) ) ) )
+	      (print-call-chain ##sys#standard-error)
+	      (flush-output ##sys#standard-error) ) ) )
 	 (lambda ()
 	   (let loop ()
 	     (saveports)
