@@ -1,6 +1,6 @@
 ;;; extras.scm - Optional non-standard extensions
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -36,31 +36,35 @@
 (declare
  (unit extras)
  (usual-integrations)
+ (disable-warning redef)
  (foreign-declare #<<EOF
 #define C_hashptr(x)   C_fix(x & C_MOST_POSITIVE_FIXNUM)
 #define C_mem_compare(to, from, n)   C_fix(C_memcmp(C_c_string(to), C_c_string(from), C_unfix(n)))
 
-#ifdef _WIN32
-# define FGETS_INTO_BUFFER  0
-static C_word fgets_into_buffer(C_word str, C_word port, C_word size) { return 0; }
-#else
-# define FGETS_INTO_BUFFER  1
-
-static C_word fgets_into_buffer(C_word str, C_word port, C_word size)
-{
-  int len, n = C_unfix(size);
+static C_word fast_read_line_from_file(C_word str, C_word port, C_word size) {
+  int n = C_unfix(size);
+  int i;
+  int c;
   char *buf = C_c_string(str);
   C_FILEPTR fp = C_port_file(port);
 
-  if(C_fgets(buf, n, fp) == NULL) return C_fix(0);
+  if ((c = getc(fp)) == EOF)
+    return C_SCHEME_END_OF_FILE;
 
-  len = C_strlen(buf);
+  ungetc(c, fp);
 
-  if(len >= n - 1 && buf[ len - 1 ] != '\n') return C_SCHEME_FALSE;
-
-  return C_fix(len);
+  for (i = 0; i < n; i++) {
+    c = getc(fp);
+    switch (c) {
+    case '\r':
+      if ((c = getc(fp)) != '\n') ungetc(c, fp);
+    case EOF: clearerr(fp);
+    case '\n': return C_fix(i);
+    }
+    buf[i] = c;
+  }
+  return C_SCHEME_FALSE;
 }
-#endif
 EOF
 ) )
 
@@ -69,6 +73,7 @@ EOF
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
      ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string ##sys#substring
      ##sys#for-each ##sys#map ##sys#setslot ##sys#allocate-vector ##sys#check-pair ##sys#not-a-proper-list-error 
@@ -76,7 +81,14 @@ EOF
      ##sys#check-symbol ##sys#check-vector ##sys#floor ##sys#ceiling ##sys#truncate ##sys#round 
      ##sys#check-number ##sys#cons-flonum
      ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg ##sys#print ##sys#check-structure 
-     ##sys#make-structure
+     ##sys#make-structure make-parameter hash-table-set! ##sys#hash-new-len hash-table-ref 
+     hash-table-update! floor input-port? make-vector list->vector sort! merge! open-output-string
+     get-output-string current-output-port ##sys#flush-output ##sys#write-char-0 newline
+     ##sys#number->string display write ##sys#fragments->string list->string make-string string
+     pretty-print-width ##sys#symbol->qualified-string ##extras#reverse-string-append ##sys#number?
+     ##sys#procedure->string ##sys#pointer->string port? ##sys#user-print-hook char-name 
+     read open-input-string ##sys#peek-char-0 ##sys#read-char-0 ##sys#write-char call-with-input-file
+     read-line reverse make-string ##sys#string-append random
      ##sys#gcd ##sys#lcm ##sys#fudge ##sys#check-list ##sys#user-read-hook) ) ] )
 
 #{extras
@@ -99,7 +111,8 @@ EOF
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "extras.exports")) ] )
 
 
 (register-feature! 'extras)
@@ -113,7 +126,7 @@ EOF
 	[call-with-input-file call-with-input-file] )
     (lambda (#!optional (port ##sys#standard-input) (reader read) max)
       (define (slurp port)
-	(do ((x (read port) (reader port))
+	(do ((x (reader port) (reader port))
 	     (i 0 (fx+ i 1))
 	     (xs '() (cons x xs)) )
 	    ((or (eof-object? x) (and max (fx>= i max))) (reverse xs)) ) )
@@ -192,7 +205,7 @@ EOF
 
 ;;; List operators:
 
-(define (atom? x) (##core#inline "C_i_atomp" x))
+(define (atom? x) (##core#inline "C_i_not_pair_p" x))
 
 (define (tail? x y)
   (##sys#check-list y 'tail?)
@@ -356,9 +369,6 @@ EOF
 
 ;;; Line I/O:
 
-
-(define-foreign-variable FGETS_INTO_BUFFER bool)
-
 (define read-line
   (let ([make-string make-string])
     (define (fixup str len)
@@ -371,44 +381,51 @@ EOF
       (let* ([parg (pair? args)]
 	     [p (if parg (car args) ##sys#standard-input)]
 	     [limit (and parg (pair? (cdr args)) (cadr args))]
-	     [buffer (make-string 256)]
-	     [len 256] )
+	     [buffer-len (if limit limit 256)]
+	     [buffer (make-string buffer-len)])
 	(##sys#check-port p 'read-line)
-	(if (and FGETS_INTO_BUFFER (eq? 'stream (##sys#slot p 7)))
-	    (let loop ([len 256] [buffer buffer] [result ""] [f #f])
-	      (let ([n (##core#inline "fgets_into_buffer" buffer p len)])
-		(cond [(eq? 0 n) (if f result #!eof)]
-		      [(not n)
-		       (loop (fx* len 2) (make-string (fx* len 2))
-			     (##sys#string-append result (##sys#substring buffer 0 (fx- len 1)))
-			     #t) ]
-		      [f (##sys#string-append result (fixup buffer (fx- n 1)))]
-		      [else (fixup buffer (fx- n 1))] ) ) )
-	    (##sys#call-with-current-continuation
-	     (lambda (return)
-	       (let loop ([i 0])
-		 (if (and limit (fx>= i limit))
-		     (return (##sys#substring buffer 0 i))
-		     (let ([c (##sys#read-char-0 p)])
-		       (if (eof-object? c)
-			   (if (fx= i 0)
-			       c
-			       (##sys#substring buffer 0 i) ) 
-			   (case c
-			     [(#\newline) (return (##sys#substring buffer 0 i))]
-			     [(#\return)
-			      (let ([c (##sys#read-char-0 p)])
-				(if (char=? c #\newline)
-				    (return (##sys#substring buffer 0 i))
-				    (begin
-				      (##core#inline "C_setsubchar" buffer i c)
-				      (loop (fx+ i 1)) ) ) ) ]
-			     [else
-			      (when (fx>= i len)
-				(set! buffer (##sys#string-append buffer (make-string len)))
-				(set! len (fx+ len len)) )
-			      (##core#inline "C_setsubchar" buffer i c)
-			      (loop (fx+ i 1)) ] ) ) ) ) ) ) ) ) ) ) ) ) 
+	(if (eq? 'stream (##sys#slot p 7))
+	    (begin
+	      (let loop ([len buffer-len]
+			 [buffer buffer]
+			 [result ""]
+			 [f #f])
+		(let ([n (##core#inline "fast_read_line_from_file" buffer p len)])
+		  (cond [(eof-object? n) (if f result #!eof)]
+			[(and limit (not n))
+			 (##sys#string-append result (##sys#substring buffer 0 limit))]
+			[(not n)
+			 (loop (fx* len 2) (make-string (fx* len 2))
+			       (##sys#string-append result
+						    (##sys#substring buffer 0 len))
+			       #t) ]
+			[f (##sys#setslot p 4 (fx+ (##sys#slot p 4) 1))
+			   (##sys#string-append result (##sys#substring buffer 0 n))]
+			[else
+			 (##sys#setslot p 4 (fx+ (##sys#slot p 4) 1))
+			 (##sys#substring buffer 0 n)] ) ) ))
+	    (let loop ([i 0])
+	      (if (and limit (fx>= i limit))
+		  (##sys#substring buffer 0 i)
+		  (let ([c (##sys#read-char-0 p)])
+		    (if (eof-object? c)
+			(if (fx= i 0)
+			    c
+			    (##sys#substring buffer 0 i) ) 
+			(case c
+			  [(#\newline) (##sys#substring buffer 0 i)]
+			  [(#\return)
+			   (let ([c (peek-char p)])
+			     (if (char=? c #\newline)
+				 (begin (##sys#read-char-0 p)
+					(##sys#substring buffer 0 i))
+				 (##sys#substring buffer 0 i) ) ) ]
+			  [else
+			   (when (fx>= i buffer-len)
+				 (set! buffer (##sys#string-append buffer (make-string buffer-len)))
+				 (set! buffer-len (fx+ buffer-len buffer-len)) )
+			   (##core#inline "C_setsubchar" buffer i c)
+			   (loop (fx+ i 1)) ] ) ) ) ) ) ) ) ) ) )
 
 (define read-lines
   (let ([read-line read-line]
@@ -1176,6 +1193,20 @@ EOF
 	    [else (cons (##sys#substring str pos (fx+ pos len)) (loop (fx- total len) (fx+ pos len)))] ) ) ) )
 	   
 
+;;; Remove suffix
+
+(define (string-chomp str #!optional (suffix "\n"))
+  (##sys#check-string str 'string-chomp)
+  (##sys#check-string suffix 'string-chomp)
+  (let* ((len (##sys#size str))
+	 (slen (##sys#size suffix)) 
+	 (diff (fx- len slen)) )
+    (if (and (fx>= len slen)
+	     (##core#inline "C_substring_compare" str suffix diff 0 slen) )
+	(##sys#substring str 0 diff)
+	str) ) )
+
+
 ;;; Write simple formatted output:
 
 (define fprintf
@@ -1185,49 +1216,49 @@ EOF
     (lambda (port msg . args)
       (let rec ([msg msg] [args args])
 	(##sys#check-string msg 'fprintf)
+	(##sys#check-port port 'fprintf)
 	(let ((index 0)
 	      (len (##sys#size msg)) )
-
 	  (define (fetch)
 	    (let ((c (##core#inline "C_subchar" msg index)))
-	      (set! index (##core#inline "C_fixnum_plus" index 1))
+	      (set! index (fx+ index 1))
 	      c) )
-
 	  (define (next)
 	    (if (cond-expand [unsafe #f] [else (##core#inline "C_eqp" args '())])
 		(##sys#error 'fprintf "too few arguments to formatted output procedure")
 		(let ((x (##sys#slot args 0)))
 		  (set! args (##sys#slot args 1)) 
 		  x) ) )
-
-	  (do ([c (fetch) (fetch)])
-	      ((fx> index len))
-	    (if (eq? c #\~)
-		(let ((dchar (fetch)))
-		  (case (char-upcase dchar)
-		    ((#\S) (write (next) port))
-		    ((#\A) (display (next) port))
-		    ((#\C) (##sys#write-char-0 (next) port))
-		    ((#\B) (display (##sys#number->string (next) 2) port))
-		    ((#\O) (display (##sys#number->string (next) 8) port))
-		    ((#\X) (display (##sys#number->string (next) 16) port))
-		    ((#\!) (##sys#flush-output port))
-		    ((#\?)
-		     (let* ([fstr (next)]
-			    [lst (next)] )
-		       (##sys#check-list lst 'fprintf)
-		       (rec fstr lst) ) )
-		    ((#\~) (##sys#write-char-0 #\~ port))
-		    ((#\%) (newline port))
-		    ((#\% #\N) (newline port))
-		    (else
-		     (if (char-whitespace? dchar)
-			 (let skip ((c (fetch)))
-			   (if (char-whitespace? c)
-			       (skip (fetch))
-			       (set! index (##core#inline "C_fixnum_difference" index 1)) ) )
-			 (##sys#error 'fprintf "illegal format-string character" dchar) ) ) ) )
-		(##sys#write-char-0 c port) ) ) ) ) ) ) )
+	  (let loop ()
+	    (unless (fx>= index len)
+	      (let ((c (fetch)))
+		(if (and (eq? c #\~) (fx< index len))
+		    (let ((dchar (fetch)))
+		      (case (char-upcase dchar)
+			((#\S) (write (next) port))
+			((#\A) (display (next) port))
+			((#\C) (##sys#write-char-0 (next) port))
+			((#\B) (display (##sys#number->string (next) 2) port))
+			((#\O) (display (##sys#number->string (next) 8) port))
+			((#\X) (display (##sys#number->string (next) 16) port))
+			((#\!) (##sys#flush-output port))
+			((#\?)
+			 (let* ([fstr (next)]
+				[lst (next)] )
+			   (##sys#check-list lst 'fprintf)
+			   (rec fstr lst) ) )
+			((#\~) (##sys#write-char-0 #\~ port))
+			((#\%) (newline port))
+			((#\% #\N) (newline port))
+			(else
+			 (if (char-whitespace? dchar)
+			     (let skip ((c (fetch)))
+			       (if (char-whitespace? c)
+				   (skip (fetch))
+				   (set! index (fx- index 1)) ) )
+			     (##sys#error 'fprintf "illegal format-string character" dchar) ) ) ) )
+		    (##sys#write-char-0 c port) )
+		(loop) ) ) ) ) ) ) ) )
 
 
 (define printf
@@ -1525,7 +1556,7 @@ EOF
 			   (fx+ i 1)
 			   (fx- len 1) ) ) ) ) ) ) )
     (##sys#check-exact limit 'hash)
-    (##core#inline "C_fixnum_modulo" (fxand #x00ffffff (rechash x 0)) limit) ) )
+    (##core#inline "C_fixnum_modulo" (fxand (foreign-value "C_MOST_POSITIVE_FIXNUM" int) (rechash x 0)) limit) ) )
 
 (define (hash x #!optional (bound default-hash-bound))
   (##sys#check-exact bound 'hash)
@@ -1536,14 +1567,14 @@ EOF
 (define (string-hash s #!optional (bound default-hash-bound))
   (##core#inline 
    "C_fixnum_modulo"
-   (fxand #x00ffffff (##core#inline "C_hash_string" s))
-   limit) )
+   (##core#inline "C_hash_string" s)
+   bound) )
 
 (define (string-ci-hash s #!optional (bound default-hash-bound))
   (##core#inline 
    "C_fixnum_modulo"
-   (fxand #x00ffffff (##core#inline "C_hash_string_ci" s))
-   limit) )
+   (##core#inline "C_hash_string_ci" s)
+   bound) )
 
 
 ;;; Access:
@@ -1556,7 +1587,9 @@ EOF
   ;; This one was suggested by Sven Hartrumpf.
   (let ([eq0 eq?]
 	[floor floor] )
-    (lambda (ht key proc init)
+    (lambda (ht key proc #!optional
+	   (init (lambda () (##sys#signal-hook #:access-error 'hash-table-update!
+			    "hash-table does not contain key" key ht))))
       (##sys#check-structure ht 'hash-table 'hash-table-update!)
       (let restart ()
 	(let* ((vec (##sys#slot ht 1))
@@ -1579,7 +1612,7 @@ EOF
 		    ;; Fast path (eq? test):
 		    (let loop ((bucket bucket0))
 		      (cond ((eq? bucket '())
-			     (##sys#setslot vec k (cons (cons key (init)) bucket0))
+			     (##sys#setslot vec k (cons (cons key (proc (init))) bucket0))
 			     (##sys#setslot ht 2 c) )
 			    (else
 			     (let ((b (##sys#slot bucket 0)))
@@ -1588,7 +1621,7 @@ EOF
 				   (loop (##sys#slot bucket 1)) ) ) ) ) )
 		    (let loop ((bucket bucket0))
 		      (cond ((eq? bucket '())
-			     (##sys#setslot vec k (cons (cons key (init)) bucket0))
+			     (##sys#setslot vec k (cons (cons key (proc (init))) bucket0))
 			     (##sys#setslot ht 2 c) )
 			    (else
 			     (let ((b (##sys#slot bucket 0)))
@@ -1892,3 +1925,32 @@ EOF
 	 (if (or (not (##core#inline "C_blockp" lst))
 		 (not (##core#inline "C_pairp" lst)) )
 	     (##sys#not-a-proper-list-error lst0 'list->queue) ) ) ) ) )
+
+
+; (queue-push-back! queue item)
+; Pushes an item into the first position of a queue.
+
+(define (queue-push-back! q item)
+  (##sys#check-structure q 'queue 'queue-push-back!)
+  (let ((newlist (cons item (##sys#slot q 1))))
+    (##sys#setslot q 1 newlist)
+    (if (eq? '() (##sys#slot q 2))
+        (##sys#setslot q 2 newlist))))
+
+; (queue-push-back-list! queue item-list)
+; Pushes the items in item-list back onto the queue,
+; so that (car item-list) becomes the next removable item.
+
+(define-macro (last-pair lst0)
+  `(do ((lst ,lst0 (##sys#slot lst 1)))
+       ((eq? (##sys#slot lst 1) '()) lst)))
+
+(define (queue-push-back-list! q itemlist)
+  (##sys#check-structure q 'queue 'queue-push-back-list!)
+  (##sys#check-list itemlist 'queue-push-back-list!)
+  (let* ((newlist (append itemlist (##sys#slot q 1)))
+         (newtail (if (eq? newlist '())
+                       '()
+                       (last-pair newlist))))
+    (##sys#setslot q 1 newlist)
+    (##sys#setslot q 2 newtail)))
