@@ -133,6 +133,7 @@ The value returned is the value of the last form in BODY."
                            "define-signature"
                            "define-structure"
                            "define-syntax"
+                           "define-for-syntax"
                            "define-syntax-form"
                            "define-optimizer"
                            "define-values") 1) "\\)"
@@ -160,7 +161,7 @@ The value returned is the value of the last form in BODY."
               "else"
               "foreign-lambda*" "foreign-safe-lambda*" "foreign-primitive"
               "foreign-lambda" "foreign-safe-lambda" "foreign-code" "foreign-safe-wrapper"
-              "match" "match-lambda" "match-define" "match-let" "match-let*"
+              "match" "match-lambda" "match-lambda*" "match-define" "match-let" "match-let*"
 
               "case" "case-lambda" "cond" "cond-expand" "condition-case" "switch"
 	      "handle-exceptions"
@@ -490,9 +491,7 @@ waiting."
 (defun hen-describe-symbol (name)
  "Lookup documentation for symbol NAME."
  (interactive (hen-csi-symbol-completing-read "Describe symbol: "))
- (info-lookup-symbol name 'hen-mode) ;
- ;;(hen-lookup-info-doc name)
- )
+ (info-lookup-symbol name 'hen-mode))
 
 (defun hen-csi-apropos (regex)
  "List the symbols matching REGEX."
@@ -529,27 +528,6 @@ waiting."
      (apropos-mode)))
  (pop-to-buffer "*Chicken Apropos*" t))
 
-(defconst hen-info-doc-list '("(r5rs)Index" "(chicken)Index"))
-
-(defun hen-lookup-info-doc (topic)
- (let ((docs hen-info-doc-list)
-       (pattern (format "\n\\* +\\([^\n:]*%s[^\n:]*\\):[ \t]*\\([^.\n]*\\)\\.[ \t]*\\([0-9]*\\)"
-                        topic))
-       doc node found)
-   (while (and (consp docs) (not found))
-     (setq doc (car docs)
-           docs (cdr docs))
-     (setq found (save-window-excursion
-                   (save-excursion
-                     (Info-goto-node doc)
-                     (goto-char (point-min))
-                     (re-search-forward pattern nil t)))))
-   (if found
-       (progn
-         (pop-to-buffer "*info*")
-         (hen-lookup-info-doc topic))
-     (error "Can't find documentation for %s" topic))))
-
 (info-lookup-add-help
 :mode 'hen-mode
 :regexp "[^()'\" \t\n]+"
