@@ -1,6 +1,6 @@
 ;;;; lolevel.scm - Low-level routines for CHICKEN
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -37,7 +37,8 @@
   (unit lolevel)
   (uses extras)
   (usual-integrations)
-  (hide ipc-hook xproc-tag)
+  (disable-warning var redef)
+  (hide ipc-hook-0 xproc-tag)
   (foreign-declare #<<EOF
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 # include <sys/types.h>
@@ -64,7 +65,14 @@ EOF
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
+     ##sys#symbol-hash-toplevel-binding? ##sys#make-locative ##sys#become! make-hash-table 
+     hash-table-ref/default ##sys#make-string make-vector hash-table-set! hash-table-set!
+     make-property-condition make-composite-condition signal ##sys#set-pointer-address! ##sys#make-vector
+     ##sys#make-pointer byte-vector-fill! make-string make-byte-vector ##sys#not-a-proper-list-error ##sys#check-pointer
+     ##sys#locative? ##sys#bytevector?
+     extend-procedure ##sys#lambda-decoration ##sys#decorate-lambda ##sys#make-tagged-pointer ##sys#check-special
      ##sys#vector->closure! ##sys#error ##sys#signal-hook ##sys#address->pointer ##sys#pointer->address) ) ] )
 
 (cond-expand
@@ -83,7 +91,8 @@ EOF
     (define-macro (##sys#check-pointer . _) '(##core#undefined))
     (define-macro (##sys#check-special . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "lolevel.exports"))] )
 
 (register-feature! 'lolevel)
 
@@ -680,15 +689,17 @@ EOF
 
 ;;; Hooks:
 
-(define ipc-hook #f)			; we need this because `##sys#invalid-procedure-call-hook' may not have free variables.
+(define ipc-hook-0 #f)			; we need this because `##sys#invalid-procedure-call-hook' may not have free variables.
 
-(define (invalid-procedure-call-handler proc)
+(define (set-invalid-procedure-call-handler! proc)
   (unless (procedure? proc)
-    (##sys#signal-hook #:type-error 'invalid-procedure-call-handler "bad argument type - not a procedure" proc) )
-  (set! ipc-hook proc)
+    (##sys#signal-hook #:type-error 'set-invalid-procedure-call-handler! "bad argument type - not a procedure" proc) )
+  (set! ipc-hook-0 proc)
   (set! ##sys#invalid-procedure-call-hook 
     (lambda args
-      (ipc-hook ##sys#last-invalid-procedure args) ) ) )
+      (ipc-hook-0 ##sys#last-invalid-procedure args) ) ) )
+
+(define invalid-procedure-call-handler set-invalid-procedure-call-handler!) ; DEPRECATED
 
 (define (unbound-variable-value . val)
   (set! ##sys#unbound-variable-value-hook 
