@@ -1,6 +1,6 @@
 ;;;; optimizer.scm - The CHICKEN Scheme compiler (optimizations)
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -41,12 +41,12 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home decompose-lambda-list external-to-pointer
-  foreign-type-table-size copy-node! export-list inline-list not-inline-list
+  copy-node! export-list inline-list not-inline-list
   unit-name insert-timer-checks used-units external-variables
   debug-info-index debug-info-vector-name profile-info-vector-name
-  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
+  foreign-declarations emit-trace-info block-compilation line-number-database-size
   always-bound-to-procedure block-globals make-block-variable-literal block-variable-literal? block-variable-literal-name
-  target-heap-size target-stack-size try-harder default-installation-home 
+  target-heap-size target-stack-size constant-declarations
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
@@ -55,10 +55,10 @@
   direct-call-ids foreign-type-table first-analysis expand-debug-lambda expand-debug-assignment expand-debug-call
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database! scan-toplevel-assignments
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
-  reorganize-recursive-bindings substitution-table simplify-named-call
+  reorganize-recursive-bindings substitution-table simplify-named-call compiler-warning
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! expand-foreign-callback-lambda* debug-lambda-list debug-variable-list debugging
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -79,7 +79,8 @@
   (match-error-control #:fail) )
 
 (include "tweaks")
-(include "parameters")
+
+(define-constant maximal-number-of-free-variables-for-liftable 16)
 
 
 ;;; Scan toplevel expressions for assignments:
@@ -202,8 +203,7 @@
 	       ((##core#call)
 		(if (eq? '##core#variable (node-class (car subs)))
 		    (let ((var (first (node-parameters (car subs)))))
-		      (if (and try-harder
-			       (or (test var 'standard-binding)
+		      (if (and (or (test var 'standard-binding)
 				   (test var 'extended-binding) )
 			       (test var 'foldable)
 			       (every constant-node? (cddr subs)) )
@@ -274,7 +274,7 @@
 				      [else (build-lambda-list used argc rest)] )
 				(fourth params) )
 			  (list (walk (first subs))) ) ) ) ) ]
-		   [(and try-harder (test (first params) 'explicit-rest))
+		   [(test (first params) 'explicit-rest)
 		    (decompose-lambda-list
 		     llist
 		     (lambda (vars argc rest)
@@ -305,6 +305,22 @@
 			   (debugging 'o "contracted procedure" var)
 			   (touch)
 			   (walk (inline-lambda-bindings llist args (first (node-subexpressions lval)) #f)) ) ]
+			[(memq var constant-declarations)
+			 (or (and-let* ((k (car args))
+					((eq? '##core#variable (node-class k)))
+					(kvar (first (node-parameters k)))
+					(lval (and (not (test kvar 'unknown)) (test kvar 'value))) 
+					(eq? '##core#lambda (node-class lval))
+					(llist (third (node-parameters lval)))
+					((or (test (car llist) 'unused)
+					     (and (not (test (car llist) 'references))
+						  (not (test (car llist) 'assigned)))))
+					((not (any (cut expression-has-side-effects? <> db) (cdr args) ))))
+			       (debugging 'x "removed call to constant procedure with unused result" var)
+			       (make-node
+				'##core#call '(#t)
+				(list k (make-node '##core#undefined '() '())) ) ) 
+			     (walk-generic n class params subs)) ]
 			[(and lval (eq? '##core#lambda (node-class lval)))
 			 (let* ([lparams (node-parameters lval)]
 				[llist (third lparams)] )
@@ -385,7 +401,7 @@
 		    (make-node '##core#undefined '() '()) ]
 		   [(and (or (not (test var 'global))
 			     block-compilation
-			     (and try-harder export-list (not (memq var export-list))) )
+			     (and export-list (not (memq var export-list))) )
 			 (not (test var 'references)) 
 			 (not (expression-has-side-effects? (first subs) db)) )
 		    (touch)
@@ -401,7 +417,7 @@
 	    n
 	    (make-node class params subs2) ) ) )
 
-    (if (and try-harder (perform-pre-optimization! node db))
+    (if (perform-pre-optimization! node db)
 	(values node #t)
 	(begin
 	  (debugging 'p "traversal phase...")
@@ -618,8 +634,7 @@
 	(let ([c (node-class body)]
 	      [params (node-parameters body)] 
 	      [subs (node-subexpressions body)] )
-	  (and try-harder
-	       (eq? c 'let)
+	  (and (eq? c 'let)
 	       (null? (cdr params))
 	       (let* ([val (first subs)]
 		      [valparams (node-parameters val)]
@@ -1355,7 +1370,9 @@
 			     (set! ksites (alist-cons #f n ksites))
 			     (cond [(eq? kvar (first arg0p))
 				    (unless (= argc (length (cdr subs)))
-				      (warning "known procedure called recursively with wrong number of arguments: `~A'" fnvar) )
+				      (compiler-warning
+				       'call
+				       "known procedure called recursively with wrong number of arguments: `~A'" fnvar) )
 				    (node-class-set! n '##core#recurse)
 				    (node-parameters-set! n (list #t id))
 				    (node-subexpressions-set! n (cddr subs)) ]
@@ -1364,7 +1381,9 @@
 					 (let* ([klam (cdr a)]
 						[kbody (first (node-subexpressions klam))] )
 					   (unless (= argc (length (cdr subs)))
-					     (warning "known procedure called recursively with wrong number of arguments: `~A'" fnvar) )
+					     (compiler-warning
+					      'call
+					      "known procedure called recursively with wrong number of arguments: `~A'" fnvar) )
 					   (node-class-set! n 'let)
 					   (node-parameters-set! n (take (third (node-parameters klam)) 1))
 					   (node-subexpressions-set!
@@ -1394,7 +1413,9 @@
 	      (let* ([n (cdr site)]
 		     [nsubs (node-subexpressions n)] )
 		(unless (= argc (length (cdr nsubs)))
-		  (warning "known procedure called with wrong number of arguments: `~A'" fnvar) )
+		  (compiler-warning 
+		   'call 
+		   "known procedure called with wrong number of arguments: `~A'" fnvar) )
 		(node-subexpressions-set!
 		 n
 		 (list (second nsubs)
