@@ -1,6 +1,6 @@
 ;;;; pcre.scm - Unit for using the PCRE regex package
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -59,7 +59,7 @@ EOF
  [else
   (declare
     (no-bound-checks)
-    ;(no-procedure-checks)
+    (no-procedure-checks-for-usual-bindings)
     ) ] )
 
 (cond-expand
@@ -76,7 +76,8 @@ EOF
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "pcre.exports")) ] )
 
 (register-feature! 'regex 'pcre)
 
