@@ -2,7 +2,7 @@
 ;
 ; By Sergey Khorev
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -74,9 +74,14 @@
 #ifndef WIN32_LEAN_AND_MEAN
 # define WIN32_LEAN_AND_MEAN
 #endif
-#include <windows.h>
 
-#if _MSC_VER > 1300
+/*
+MinGW should have winsock2.h and ws2tcpip.h as well.
+The CMake build will set HAVE_WINSOCK2_H and HAVE_WS2TCPIP_H.
+However, the _MSC_VER test is still needed for vcbuild.bat.
+./configure doesn't test for these.  It should, for MinGW.
+*/
+#if (_MSC_VER > 1300) || (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #else
@@ -588,6 +593,7 @@ EOF
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
      ##sys#make-port ##sys#file-info ##sys#update-errno ##sys#fudge ##sys#make-c-string ##sys#check-port 
      ##sys#error ##sys#signal-hook ##sys#peek-unsigned-integer
@@ -607,7 +613,8 @@ EOF
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "posixwin.exports"))] )
 
 (register-feature! 'posix)
 
