@@ -29,6 +29,7 @@
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
      ##sys#check-string ##sys#check-exact ##sys#make-pointer ##sys#cons ##sys#size ##sys#slot) ) ] )
 
@@ -46,7 +47,8 @@
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "pregexp.exports"))] )
 
 (register-feature! 'regex 'pregexp)
 
