@@ -1,6 +1,6 @@
 ;;;; profiler.scm - Support code for profiling applications
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -44,10 +44,12 @@
  [paranoia]
  [else
   (declare
+    (bound-to-procedure
+     write-char write make-vector)
     (no-bound-checks) ) ] )
 
 
-(include "parameters.scm")
+(define-constant profile-info-entry-size 5)
 
 
 ;;; Globals:
@@ -79,6 +81,9 @@
 	(set! ##sys#profile-vector-list (cons vec ##sys#profile-vector-list))
 	vec) ) ) )
 
+(define (##sys#set-profile-info-vector! vec i x)
+  (##sys#setslot vec (* i profile-info-entry-size) x) )
+
 
 ;;; Entry and exit into/out of profiled lambda:
 
