@@ -7,6 +7,7 @@
 (declare
   (unit srfi-1)
   (disable-interrupts)
+  (disable-warning redef)
   (hide ##srfi1#cars+cdrs/no-test ##srfi1#cdrs ##srfi1#cars+ ##srfi1#really-append-map ##srfi1#cars+cdrs+
 	##srfi1#cars+cdrs ##srfi1#lset2<=)
   (extended-bindings)
@@ -32,6 +33,11 @@
  [paranoia]
  [else
   (declare
+    (no-procedure-checks-for-usual-bindings)
+    (bound-to-procedure 
+     every any partition! reduce lset-difference! append! pair-fold lset-diff+intersection! fold
+     lset-difference filter! filter delete span! span find-tail find delete! pair-for-each car+cdr
+     reduce-right last-pair drop)
     (no-bound-checks) ) ] )
 
 (cond-expand
@@ -48,7 +54,8 @@
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "srfi-1.exports"))] )
 
 (register-feature! 'srfi-1)
 
@@ -450,20 +457,19 @@
 
 (define (list= = . lists)
   (or (null? lists) ; special case
-
       (let lp1 ((list-a (car lists)) (others (cdr lists)))
 	(or (null? others)
 	    (let ((list-b (car others))
 		  (others (cdr others)))
 	      (if (eq? list-a list-b)	; EQ? => LIST=
 		  (lp1 list-b others)
-		  (let lp2 ((list-a list-a) (list-b list-b))
-		    (if (null-list? list-a)
-			(and (null-list? list-b)
+		  (let lp2 ((la list-a) (lb list-b))
+		    (if (null-list? la)
+			(and (null-list? lb)
 			     (lp1 list-b others))
-			(and (not (null-list? list-b))
-			     (= (car list-a) (car list-b))
-			     (lp2 (cdr list-a) (cdr list-b)))))))))))
+			(and (not (null-list? lb))
+			     (= (car la) (car lb))
+			     (lp2 (cdr la) (cdr lb)))))))))))
 			
 
 
