@@ -5,6 +5,7 @@
   (unit srfi-13)
   (uses srfi-14)
   (fixnum)
+  (disable-warning redef)
   (hide %string-prefix? %string-hash %finish-string-concatenate-reverse %string-suffix-length %string-prefix-length
 	%string-map %string-copy! %string-compare %substring/shared %string-suffix? %multispan-repcopy!
 	%string-prefix-length-ci %string-suffix-length-ci %string-prefix-ci? %string-suffix-ci?
@@ -34,6 +35,13 @@
  [paranoia]
  [else
   (declare
+    (no-procedure-checks-for-usual-bindings)
+    (bound-to-procedure
+     string-concatenate check-substring-spec ##srfi13#string-fill! string-parse-final-start+end
+     ##sys#substring string-index-right string-skip-right substring/shared
+     string-concatenate/shared make-kmp-restart-vector string-ci= string= char-set?
+     char-set-contains? string-fold char-set string-skip string-index string-downcase! char->int
+     string-parse-start+end substring-spec-ok?)
     (no-bound-checks) ) ] )
 
 (cond-expand
@@ -50,7 +58,8 @@
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-bytevector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "srfi-13.exports"))] )
 
 (register-feature! 'srfi-13)
 
@@ -265,6 +274,7 @@
 ;	       start substring/shared)
     (let ([n (:optional maybe-end slen)])
       (##sys#check-exact n 'substring/shared)
+      (check-substring-spec 'substring/shared s start n)
       (%substring/shared s start n) ) ) )
 #|
     (%substring/shared s start
@@ -1331,6 +1341,7 @@
 ;;; Library-internal routine
 (define (%string-copy! to tstart from fstart fend)
   (if (> fstart tstart)
+      ;; Might be better: (##core#inline "C_substring_copy" from to fstart fend tstart)
       (do ((i fstart (+ i 1))
 	   (j tstart (+ j 1)))
 	  ((>= i fend))
