@@ -1,6 +1,6 @@
-;;;; srfi-18.scm - Simple thread unit - felix
+;;; srfi-18.scm - Simple thread unit - felix
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -36,6 +36,7 @@
 (declare
  (unit srfi-18)
  (uses scheduler)
+ (disable-warning redef)
  (disable-interrupts)
  (usual-integrations)
  (hide ##sys#compute-time-limit) )
@@ -45,7 +46,11 @@
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
+     condition-property-accessor ##sys#tty-port? ##sys#thread-block-for-i/o thread-yield! ##sys#thread-unblock!
+     ##sys#thread-basic-unblock! gensym ##sys#thread-block-for-timeout! ##sys#thread-kill!
+     ##sys#thread-block-for-termination! make-thread ##sys#exact->inexact ##sys#flonum-fraction truncate
      ##sys#add-to-ready-queue
      ##sys#schedule ##sys#make-thread
      ##sys#check-number ##sys#error ##sys#signal-hook ##sys#signal
@@ -66,7 +71,8 @@
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-byte-vector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "srfi-18.exports"))] )
 
 (register-feature! 'srfi-18)
 
@@ -165,10 +171,12 @@ EOF
 	(##sys#setslot 
 	 thread 1 
 	 (lambda () 
-	   (let ((result (thunk)))
-	     (##sys#setslot thread 2 result)
-	     (##sys#thread-kill! thread 'dead)
-	     (##sys#schedule) ) ) )
+	   (##sys#call-with-values
+	    thunk
+	    (lambda results
+	      (##sys#setslot thread 2 results)
+	      (##sys#thread-kill! thread 'dead)
+	      (##sys#schedule) ) ) ) )
 	thread) ) ) )
 
 (define (thread? x) (##sys#structure? x 'thread))
@@ -233,16 +241,19 @@ EOF
 	    ct 1
 	    (lambda ()
 	      (case (##sys#slot thread 3)
-		[(dead) (return (##sys#slot thread 2))]
+		[(dead) (apply return (##sys#slot thread 2))]
 		[(terminated)
 		 (return 
 		  (##sys#signal
-		   (##sys#make-structure 'condition '(uncaught-exception) (list 'reason (##sys#slot thread 7)) ) ) ) ]
+		   (##sys#make-structure 
+		    'condition '(uncaught-exception)
+		    (list '(uncaught-exception . reason) (##sys#slot thread 7)) ) ) ) ]
 		[else
 		 (return
 		  (if tosupplied
 		      toval
-		      (##sys#signal (##sys#make-structure 'condition '(join-timeout-exception) '())) ) ) ] ) ) )
+		      (##sys#signal
+		       (##sys#make-structure 'condition '(join-timeout-exception) '())) ) ) ] ) ) )
 	   (##sys#thread-block-for-termination! ct thread) 
 	   (##sys#schedule) ) ) ) ) ) )
 	   
@@ -250,7 +261,7 @@ EOF
   (##sys#check-structure thread 'thread 'thread-terminate!)
   (when (eq? thread ##sys#primordial-thread)
     ((##sys#exit-handler)) )
-  (##sys#setislot thread 2 (##core#undefined))
+  (##sys#setislot thread 2 (list (##core#undefined)))
   (##sys#setslot thread 7 (##sys#make-structure 'condition '(terminated-thread-exception) '()))
   (##sys#thread-kill! thread 'terminated)
   (when (eq? thread ##sys#current-thread) (##sys#schedule)) )
@@ -393,6 +404,7 @@ EOF
 	     (let* ([wt (##sys#slot waiting 0)]
 		    [wts (##sys#slot wt 3)] )
 	       (##sys#setslot mutex 3 (##sys#slot waiting 1))
+	       (##sys#setislot mutex 5 #t)
 	       (when (or (eq? wts 'blocked) (eq? wts 'sleeping))
 		 (##sys#setslot mutex 2 wt)
 		 (##sys#setslot wt 8 (cons mutex (##sys#slot wt 8)))
@@ -407,11 +419,11 @@ EOF
     (lambda name
       (##sys#make-structure
        'condition-variable 
-       (if (pair? name)
+       (if (pair? name)			; #1 name
 	   (car name)
 	   (gensym 'condition-variable) )
-       '()				; list of waiting threads
-       (##core#undefined) ) ) ) )	; specific
+       '()				; #2 list of waiting threads
+       (##core#undefined) ) ) ) )	; #3 specific
 
 (define (condition-variable? x)
   (##sys#structure? x 'condition-variable) )
@@ -449,8 +461,8 @@ EOF
 
 ;;; Change continuation of thread to signal an exception:
 
-(define (thread-deliver-signal! thread exn)
-  (##sys#check-structure thread 'thread 'thread-deliver-signal!)
+(define (thread-signal! thread exn)
+  (##sys#check-structure thread 'thread 'thread-signal!)
   (if (eq? thread ##sys#current-thread)
       (##sys#signal exn)
       (let ([old (##sys#slot thread 1)])
@@ -461,6 +473,8 @@ EOF
 	   (old) ) )
 	(##sys#thread-unblock! thread) ) ) )
 
+(define thread-deliver-signal! thread-signal!)
+
 
 ;;; Don't block in the repl: (by Chris Double)
 
