@@ -1,6 +1,6 @@
 ;;;; srfi-4.scm - Homogeneous numeric vectors
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -36,6 +36,7 @@
 (declare
  (unit srfi-4)
  (disable-interrupts)
+ (disable-warning redef)
  (usual-integrations)
  (hide ##sys#u8vector-set! ##sys#s8vector-set! ##sys#u16vector-set! ##sys#s16vector-set!
        ##sys#u32vector-set! ##sys#s32vector-set! ##sys#f32vector-set! ##sys#f64vector-set!
@@ -74,11 +75,13 @@ EOF
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
      ##sys#check-exact ##sys#u8vector-ref ##sys#u8vector-set! ##sys#s8vector-ref ##sys#s8vector-set! 
      ##sys#u16vector-ref ##sys#u16vector-set!
      ##sys#s16vector-ref ##sys#s16vector-set! ##sys#u32vector-ref ##sys#u32vector-set! ##sys#s32vector-ref 
-     ##sys#s32vector-set!
+     ##sys#s32vector-set! read list->f64vector list->s32vector list->u32vector list->u16vector list-s8vector
+     list->u8vector set-finalizer!
      ##sys#f32vector-ref ##sys#f32vector-set! ##sys#f64vector-ref ##sys#f64vector-set! ##sys#check-exact-interval
      ##sys#check-inexact-interval ##sys#check-number ##sys#check-structure ##sys#cons-flonum ##sys#check-list 
      ##sys#check-range ##sys#error ##sys#signal-hook
@@ -98,7 +101,8 @@ EOF
     (define-macro (##sys#check-port . _) '(##core#undefined))
     (define-macro (##sys#check-number . _) '(##core#undefined))
     (define-macro (##sys#check-bytevector . _) '(##core#undefined)) ) ]
- [else] )
+ [else
+  (declare (emit-exports "srfi-4.exports"))] )
 
 
 ;;; Helper routines:
@@ -580,7 +584,8 @@ EOF
   (let* ([bv (##sys#slot v 1)]
 	 [len (##sys#size bv)]
 	 [ilen (fx/ len es)] )
-    (##sys#check-range from 0 ilen loc)
+    (##sys#check-range from 0 (fx+ ilen 1) loc)
+    (##sys#check-range to 0 (fx+ ilen 1) loc)
     (let* ([size2 (fx* es (fx- to from))]
 	   [bv2 (##sys#allocate-vector size2 #t #f #t)] )
       (##core#inline "C_string_to_bytevector" bv2)
