@@ -1,6 +1,6 @@
 ;;;; support.scm - Miscellaneous support code for the CHICKEN compiler
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -42,25 +42,25 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home optimization-iterations compiler-cleanup-hook decompose-lambda-list
-  foreign-type-table-size file-io-only banner custom-declare-alist parse-easy-ffi
+  file-io-only banner custom-declare-alist disabled-warnings
   unit-name insert-timer-checks used-units source-filename pending-canonicalizations
-  foreign-declarations block-compilation analysis-database-size line-number-database-size
-  target-heap-size target-stack-size try-harder default-installation-home check-global-exports
+  foreign-declarations block-compilation line-number-database-size
+  target-heap-size target-stack-size check-global-exports check-global-imports
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  dependency-list broken-constant-nodes inline-substitutions-enabled no-c-syntax-checks
+  dependency-list broken-constant-nodes inline-substitutions-enabled
   always-bound-to-procedure block-variable-literal copy-node! valid-c-identifier? tree-copy copy-node-tree-and-rename
-  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string enable-sharp-greater-read-syntax
+  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string
   expand-profile-lambda profile-lambda-list profile-lambda-index profile-info-vector-name
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
-  reorganize-recursive-bindings substitution-table simplify-named-call check-c-syntax
+  reorganize-recursive-bindings substitution-table simplify-named-call
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! finish-foreign-result compressable-literal csc-control-file
-  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
   string->c-identifier c-ify-string words words->bytes check-and-open-input-file close-checked-input-file fold-inner
-  constant? basic-literal? source-info->string
+  constant? basic-literal? source-info->string import-table
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
   build-node-graph build-expression-tree fold-boolean inline-lambda-bindings match-node expression-has-side-effects?
@@ -68,16 +68,16 @@
   pprint-expressions-to-file foreign-type-check estimate-foreign-result-size scan-used-variables scan-free-variables
   topological-sort print-version print-usage initialize-analysis-database estimate-foreign-result-location-size
   real-name real-name-table set-real-name! real-name2 display-real-name-table display-line-number-database
-  default-declarations units-used-by-default words-per-flonum emit-control-file-item
+  default-declarations units-used-by-default words-per-flonum emit-control-file-item compiler-warning
   foreign-string-result-reserve parameter-limit eq-inline-operator optimizable-rest-argument-operators
   membership-test-operators membership-unfold-limit valid-compiler-options valid-compiler-options-with-argument
   default-optimization-iterations chop-separator chop-extension follow-without-loop dump-exported-globals
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
   foreign-argument-conversion foreign-result-conversion final-foreign-type debugging export-list block-globals
+  lookup-exports-file constant-declarations
   make-random-name foreign-type-convert-result foreign-type-convert-argument process-custom-declaration}
 
 
-(include "parameters")
 (include "tweaks")
 (include "banner")
 
@@ -87,6 +87,7 @@
 (define (compiler-cleanup-hook) #f)
 
 (define debugging-chicken '())
+(define disabled-warnings '())		; usage type load var const syntax redef use call ffi
 
 (define (bomb . msg-and-args)
   (if (pair? msg-and-args)
@@ -105,8 +106,8 @@
 	 (flush-output)
 	 #t) ) )
 
-(define (warning msg . args)		; this redefines library's `warning'
-  (when ##sys#warnings-enabled
+(define (compiler-warning class msg . args)	       
+  (when (and ##sys#warnings-enabled (not (memq class disabled-warnings)))
     (let ((out (current-error-port)))
       (apply fprintf out (string-append "Warning: " msg) args)
       (newline out) ) ) )
@@ -160,9 +161,6 @@
 	((string? x) (string->symbol x))
 	(else (string->symbol (sprintf "~a" x))) ) )
 
-(define (flonum? x)
-  (and (number? x) (not (exact? x))) )
-
 (define (build-lambda-list vars argc rest)
   (let loop ((vars vars) (n argc))
     (cond ((or (zero? n) (null? vars)) (or rest '()))
@@ -514,7 +512,9 @@
 				  (eq? 'fixnum number-type)
 				  (not (integer? c)) )
 			     (begin
-			       (warning "literal '~s' is out of range - will be truncated to integer" c)
+			       (compiler-warning
+				'type
+				"literal '~s' is out of range - will be truncated to integer" c)
 			       (inexact->exact (truncate c)) )
 			     c) ) ) )
 	       ((let)
@@ -747,18 +747,20 @@
 
 
 ;;; Some safety checks and database dumping:
-  
-(define (dump-exported-globals db)
+
+(define (dump-exported-globals db file)
   (unless block-compilation
-    (##sys#hash-table-for-each
-     (lambda (sym plist)
-       (when (and (assq 'global plist) 
-		  (assq 'assigned plist)
-		  (or (and export-list (memq sym export-list))
-		      (not (memq sym block-globals)) ) )
-	 (write sym) 
-	 (newline) ) )
-     db) ) )
+    (with-output-to-file file
+      (lambda ()
+	(##sys#hash-table-for-each
+	 (lambda (sym plist)
+	   (when (and (assq 'global plist) 
+		      (assq 'assigned plist)
+		      (or (and export-list (memq sym export-list))
+			  (not (memq sym block-globals)) ) )
+	     (write sym) 
+	     (newline) ) )
+	 db) ) ) ) )
 
 (define (dump-undefined-globals db)
   (##sys#hash-table-for-each
@@ -775,10 +777,33 @@
       (##sys#hash-table-for-each
        (lambda (sym plist)
 	 (when (and (memq sym exps) (not (assq 'assigned plist)))
-	   (warning "exported global variable `~S' is used but not defined" sym) )
+	   (compiler-warning 'var "exported global variable `~S' is used but not defined" sym) )
 	 (set! exps (delete sym exps eq?)) )
        db)
-      (for-each (cut warning "exported global variable `~S' is not defined" <>) exps) ) ) )
+      (for-each (cut compiler-warning 'var "exported global variable `~S' is not defined" <>) exps) ) ) )
+
+(define (check-global-imports db)
+  (##sys#hash-table-for-each
+   (lambda (sym plist)
+     (let ((imp (##sys#hash-table-ref import-table sym))
+	   (refs (assq 'references plist))
+	   (assgn (assq 'assigned plist)) )
+       (when (assq 'global plist)
+	 (cond (assgn
+		(when imp
+		  (compiler-warning 'redef "redefinition of imported variable `~s' from `~s'" sym imp) ) )
+	       ((and (pair? refs) (not imp) (not (keyword? sym)))
+		(compiler-warning 'var "variable `~s' used but not imported" sym) ) ) ) ) )
+   db) )
+
+(define (lookup-exports-file id)
+  (and-let* ((xfile (##sys#resolve-include-filename 
+		     (string-append (->string id) ".exports")
+		     #t #t) )
+	     ((file-exists? xfile)) )
+    (when verbose-mode 
+      (printf "loading exports file ~a ...~%" xfile) )
+    (for-each (cut ##sys#hash-table-set! import-table <> id) (read-file xfile)) ) )
 
 
 ;;; Compute general statistics from analysis database:
@@ -990,7 +1015,7 @@
 	(words->bytes 3) )
        ((unsigned-integer long integer unsigned-long integer32 unsigned-integer32)
 	(words->bytes 4) )
-       ((float double number) 
+       ((float double number integer64) 
 	(words->bytes 4) )		; possibly 8-byte aligned 64-bit double
        (else
 	(cond [(and (symbol? t) (##sys#hash-table-ref foreign-type-table t))
@@ -1042,7 +1067,7 @@
     [else
      (match type
        [((or 'instance 'instance-ref) cname sname)
-	`(##tinyclos#make-instance-from-pointer ,body ,sname) ]
+	`(##tinyclos#make-instance-from-pointer ,body ,sname) ] ;XXX eggified, needs better treatment...
        [('nonnull-instance cname sname)
 	`(make ,sname 'this ,body) ]
        [_ body] ) ] ) )
@@ -1182,25 +1207,19 @@ Usage: chicken FILENAME OPTION ...
 
     -help                       display this text and exit
     -version                    display compiler version and exit
+    -release                    print release number and exit
     -verbose                    display information on compilation progress
     -quiet                      do not display compile information
 
   File and pathname options:
 
     -output-file FILENAME       specifies output-filename, default is 'out.c'
-    -split NUMBER               split the output into smaller files
-    -split-level NUMBER         how hard the compiler should try partitioning the output
     -include-path PATHNAME      specifies alternative path for included files
     -to-stdout                  write compiled file to stdout instead of file
 
   Language options:
 
     -feature SYMBOL             register feature identifier
-    -ffi-define SYMBOL          define preprocessor macro for ``easy'' FFI parser
-    -ffi-include-path PATH      set include path for ``easy'' FFI parser
-    -ffi-no-include             don't parse include files when encountered by the FFI parser
-    -ffi                        parse and compile C/C++ code and generate Scheme bindings
-    -ffi-parse                  parse C/C++ code without including it
 
   Syntax related options:
 
@@ -1217,12 +1236,16 @@ Usage: chicken FILENAME OPTION ...
   Debugging options:
 
     -no-warnings                disable warnings
+    -disable-warning CLASS      disable specific class of warnings
     -debug-level NUMBER         set level of available debugging information
     -no-trace                   disable tracing information
     -profile                    executable emits profiling information 
     -profile-name FILENAME      name of the generated profile information file
     -accumulate-profile         executable emits profiling information in append mode
     -no-lambda-info             omit additional procedure-information
+    -emit-exports FILENAME      write exported toplevel variables to FILENAME
+    -check-imports              look for undefined toplevel variables
+    -import FILENAME            read externally exported symbols from FILENAME
 
   Optimization options:
 
@@ -1262,7 +1285,6 @@ Usage: chicken FILENAME OPTION ...
 
     -debug MODES                display debugging output for the given modes
     -compress-literals NUMBER   compile literals above threshold as strings
-    -disable-c-syntax-checks    disable syntax check of C code fragments
     -unsafe-libraries           marks the generated file as being linked
                                 with the unsafe runtime system
     -raw                        do not generate implicit init- and exit code			       
@@ -1368,55 +1390,14 @@ EOF
 
 ;;; "#> ... <#" syntax:
 
-(define (enable-sharp-greater-read-syntax)
-  (set! ##sys#user-read-hook
-    (let ([old-hook ##sys#user-read-hook])
-      (lambda (char port)
-	(if (char=? #\> char)	       
-	    (let ([_ (read-char port)]		; swallow #\>
-		  [decl #f]
-		  [parse #f]
-		  [exec #f] )
-	      (case (peek-char port)
-		[(#\!)
-		 (read-char port)
-		 (set! parse #t)
-		 (set! decl #t) ]
-		[(#\?)
-		 (read-char port)
-		 (set! parse #t) ]
-		[(#\:)
-		 (read-char port)
-		 (set! exec #t) ]
-		[(#\()
-		 (let ([head (read port)])
-		   (match head
-		     [_ (for-each
-			 (lambda (t)
-			   (case t
-			     [(declare) (set! decl #t)]
-			     [(parse) (set! parse #t)]
-			     [(execute) (set! exec #t)]
-			     [else (quit "invalid tag in `#>(...) ...<#' form" t)] ) )
-			 head) ] ) ) ]
-		[else (set! decl #t)] )
-	      (let ([text (scan-sharp-greater-string port)])
-		(check-c-syntax text)
-		`(begin
-		   ,@(if decl
-			 `((declare (foreign-declare ,text)))
-			 '() )
-		   ,@(if parse
-			 `((declare (foreign-parse ,text)))
-			 '() )
-		   ,@(if exec
-			 (let ([tmp (gensym 'code_)])
-			   `((declare 
-			       (foreign-declare
-				,(sprintf "static C_word ~A() { ~A; return C_SCHEME_UNDEFINED; }\n" tmp text) ) )
-			     (##core#inline ,(symbol->string tmp)) ) )
-			 '() ) ) ) )
-	    (old-hook char port) ) ) ) ) )
+(set! ##sys#user-read-hook
+  (let ([old-hook ##sys#user-read-hook])
+    (lambda (char port)
+      (if (char=? #\> char)	       
+	  (let* ((_ (read-char port))		; swallow #\>
+		 (text (scan-sharp-greater-string port)))
+	    `(declare (foreign-declare ,text)) )
+	  (old-hook char port) ) ) ) )
 
 (define (scan-sharp-greater-string port)
   (let ([out (open-output-string)])
@@ -1438,8 +1419,6 @@ EOF
 	       (write-char c out)
 	       (loop) ] ) ) ) ) )
 
-(enable-sharp-greater-read-syntax)
-
 
 ;;; Custom declarations:
 
