@@ -1,6 +1,6 @@
 ;;;; tcp.scm - Networking stuff
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -40,9 +40,12 @@
   (fixnum-arithmetic)
   (no-bound-checks)
   (export tcp-close tcp-listen tcp-connect tcp-accept tcp-accept-ready? ##sys#tcp-port->fileno tcp-listener? tcp-addresses
-	  tcp-abandon-port tcp-listener-port tcp-listener-fileno tcp-port-numbers)
+	  tcp-abandon-port tcp-listener-port tcp-listener-fileno tcp-port-numbers tcp-buffer-size)
+  (no-procedure-checks-for-usual-bindings)
   (bound-to-procedure
-   ##net#socket ##net#bind ##net#connect ##net#listen ##net#accept 
+   ##net#socket ##net#bind ##net#connect ##net#listen ##net#accept make-parameter ##sys#string-append ##sys#tcp-port->fileno
+   ##sys#check-port ##sys#port-data ##sys#thread-block-for-i/o! make-string make-input-port make-output-port ##sys#substring
+   substring ##sys#make-c-string ##sys#schedule
    ##net#close ##net#recv ##net#send ##net#select ##net#select-write ##net#gethostaddr ##net#io-ports ##sys#update-errno
    ##sys#error ##sys#signal-hook ##net#getservbyname ##net#parse-host ##net#fresh-addr
    ##net#bind-socket ##net#shutdown)
@@ -55,6 +58,7 @@
 # else
 # include <winsock.h>
 # endif
+/* Beware: winsock2.h must come BEFORE windows.h */
 # define socklen_t       int
 static WSADATA wsa;
 # define fcntl(a, b, c)  0
@@ -92,7 +96,7 @@ EOF
 (register-feature! 'tcp)
 
 (cond-expand
- [unsafe
+ (unsafe
   (eval-when (compile)
     (define-macro (##sys#check-structure x y . _) '(##core#undefined))
     (define-macro (##sys#check-range x y z) '(##core#undefined))
@@ -104,8 +108,9 @@ EOF
     (define-macro (##sys#check-exact x) '(##core#undefined))
     (define-macro (##sys#check-port x) '(##core#undefined))
     (define-macro (##sys#check-number x) '(##core#undefined))
-    (define-macro (##sys#check-byte-vector x) '(##core#undefined)) ) ]
- [else] )
+    (define-macro (##sys#check-byte-vector x) '(##core#undefined)) ) )
+ (else
+  (declare (emit-exports "tcp.exports"))) )
 
 (define-foreign-variable errno int "errno")
 (define-foreign-variable strerror c-string "strerror(errno)")
@@ -136,13 +141,13 @@ EOF
 (define ##net#connect (foreign-lambda int "connect" int scheme-pointer int))
 
 (define ##net#make-nonblocking
-  (foreign-lambda* bool ([int fd])
+  (foreign-lambda* bool ((int fd))
     "int val = fcntl(fd, F_GETFL, 0);"
     "if(val == -1) return(0);"
     "return(fcntl(fd, F_SETFL, val | O_NONBLOCK) != -1);") )
 
 (define ##net#getsockname 
-  (foreign-lambda* c-string ([int s])
+  (foreign-lambda* c-string ((int s))
     "struct sockaddr_in sa;"
     "unsigned char *ptr;"
     "int len = sizeof(struct sockaddr_in);"
@@ -152,21 +157,21 @@ EOF
     "return(addr_buffer);") )
 
 (define ##net#getsockport
-  (foreign-lambda* int ([int s])
+  (foreign-lambda* int ((int s))
     "struct sockaddr_in sa;"
     "int len = sizeof(struct sockaddr_in);"
     "if(getsockname(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) return(-1);"
     "else return(ntohs(sa.sin_port));") )
 
 (define ##net#getpeerport
- (foreign-lambda* int ([int s])
+ (foreign-lambda* int ((int s))
    "struct sockaddr_in sa;"
    "int len = sizeof(struct sockaddr_in);"
    "if(getpeername(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) return(-1);"
    "else return(ntohs(sa.sin_port));") )
 
 (define ##net#getpeername 
-  (foreign-lambda* c-string ([int s])
+  (foreign-lambda* c-string ((int s))
     "struct sockaddr_in sa;"
     "unsigned char *ptr;"
     "unsigned int len = sizeof(struct sockaddr_in);"
@@ -238,18 +243,18 @@ EOF
        (##sys#schedule) ) ) ) )
 
 (define ##net#parse-host
-  (let ([substring substring])
+  (let ((substring substring))
     (lambda (host proto)
-      (let ([len (##sys#size host)])
-	(let loop ([i 0])
+      (let ((len (##sys#size host)))
+	(let loop ((i 0))
 	  (if (fx>= i len)
 	      (values host #f)
-	      (let ([c (##core#inline "C_subchar" host i)])
+	      (let ((c (##core#inline "C_subchar" host i)))
 		(if (char=? c #\:)		    
 		    (values
 		     (substring host (add1 i) len)
-		     (let* ([s (substring host 0 i)]
-			    [p (##net#getservbyname s proto)] )
+		     (let* ((s (substring host 0 i))
+			    (p (##net#getservbyname s proto)) )
 		       (when (eq? 0 p)
 			 (##sys#update-errno)
 			 (##sys#signal-hook
@@ -268,7 +273,7 @@ EOF
 
 (define (##net#bind-socket port style host)
   (##sys#check-exact port)
-  (let ([s (##net#socket _af_inet style 0)])
+  (let ((s (##net#socket _af_inet style 0)))
     (when (eq? _invalid_socket s)
       (##sys#update-errno)
       (##sys#error "can not create socket") )
@@ -279,12 +284,12 @@ EOF
 		   s) )
       (##sys#update-errno)
       (##sys#signal-hook #:network-error 'tcp-listen (##sys#string-append "error while setting up socket - " strerror) s) )
-    (let ([addr (make-string _sockaddr_in_size)])
+    (let ((addr (make-string _sockaddr_in_size)))
       (if host
 	  (unless (##net#gethostaddr addr host port)
 	    (##sys#signal-hook #:network-error 'tcp-listen "getting listener host IP failed - " host port) )
 	  (##net#fresh-addr addr port) )
-      (let ([b (##net#bind s addr _sockaddr_in_size)])
+      (let ((b (##net#bind s addr _sockaddr_in_size)))
 	(when (eq? -1 b)
 	  (##sys#update-errno)
 	  (##sys#signal-hook #:network-error 'tcp-listen (##sys#string-append "can not bind to socket - " strerror) s port) )
@@ -293,10 +298,10 @@ EOF
 (define-constant default-backlog 10)
 
 (define (tcp-listen port . more)
-  (let-optionals more ([w default-backlog] [host #f])
-    (let-values ([(s addr) (##net#bind-socket port _sock_stream host)])
+  (let-optionals more ((w default-backlog) (host #f))
+    (let-values (((s addr) (##net#bind-socket port _sock_stream host)))
       (##sys#check-exact w)
-      (let ([l (##net#listen s w)])
+      (let ((l (##net#listen s w)))
 	(when (eq? -1 l)
 	  (##sys#update-errno)
 	  (##sys#signal-hook #:network-error 'tcp-listen (##sys#string-append "can not listen on socket - " strerror) s port) )
@@ -308,7 +313,7 @@ EOF
 
 (define (tcp-close tcpl)
   (##sys#check-structure tcpl 'tcp-listener)
-  (let ([s (##sys#slot tcpl 1)])
+  (let ((s (##sys#slot tcpl 1)))
     (when (fx= -1 (##net#close s))
       (##sys#update-errno)
       (##sys#signal-hook #:network-error 'tcp-close (##sys#string-append "can not close TCP socket - " strerror) tcpl) ) ) )
@@ -316,26 +321,31 @@ EOF
 (define-constant +input-buffer-size+ 1024)
 (define-constant +output-chunk-size+ 8192)
 
+(define tcp-buffer-size (make-parameter #f))
+
 (define ##net#io-ports
-  (let ([make-input-port make-input-port]
-	[make-output-port make-output-port] 
-	[make-string make-string] )
+  (let ((make-input-port make-input-port)
+	(make-output-port make-output-port) 
+	(tbs tcp-buffer-size)
+	(make-string make-string) )
     (lambda (fd)
       (unless (##net#make-nonblocking fd)
 	(##sys#update-errno)
 	(##sys#signal-hook #:network-error (##sys#string-append "can not create TCP ports - " strerror)) )
-      (let* ([buf (make-string +input-buffer-size+)]
-	     [data (vector fd #f #f)]
-	     [buflen 0]
-	     [bufindex 0]
-	     [iclosed #f] 
-	     [oclosed #f]
-	     [in
+      (let* ((buf (make-string +input-buffer-size+))
+	     (data (vector fd #f #f))
+	     (buflen 0)
+	     (bufindex 0)
+	     (iclosed #f) 
+	     (oclosed #f)
+	     (outbufsize (tbs))
+	     (outbuf (and outbufsize (fx> outbufsize 0) ""))
+	     (in
 	      (make-input-port
 	       (lambda ()
 		 (when (fx>= bufindex buflen)
-		   (let ([n (let loop ()
-			      (let ([n (##net#recv fd buf +input-buffer-size+ 0)])
+		   (let ((n (let loop ()
+			      (let ((n (##net#recv fd buf +input-buffer-size+ 0)))
 				(if (eq? -1 n)
 				    (if (eq? errno _ewouldblock) 
 					(begin
@@ -348,17 +358,17 @@ EOF
 					   #:network-error
 					   (##sys#string-append "can not read from socket - " strerror) 
 					   fd) ) )
-				    n) ) ) ] )
+				    n) ) ) ) )
 		     (set! buflen n)
 		     (set! bufindex 0) ) )
 		 (if (fx>= bufindex buflen)
 		     #!eof
-		     (let ([c (##core#inline "C_subchar" buf bufindex)])
+		     (let ((c (##core#inline "C_subchar" buf bufindex)))
 		       (set! bufindex (fx+ bufindex 1))
 		       c) ) )
 	       (lambda ()
 		 (or (fx< bufindex buflen)
-		     (let ([f (##net#select fd)])
+		     (let ((f (##net#select fd)))
 		       (when (eq? f -1)
 			 (##sys#update-errno)
 			 (##sys#signal-hook #:network-error (##sys#string-append "can not check socket for input - " strerror) 
@@ -371,36 +381,52 @@ EOF
 		   (when (and oclosed (eq? -1 (##net#close fd)))
 		     (##sys#update-errno)
 		     (##sys#signal-hook #:network-error (##sys#string-append "can not close socket input port - " strerror)
-					fd) ) ) ) ) ]
-	     [out
+					fd) ) ) ) ) )
+	     (output
+	      (lambda (s)
+		(let loop ((len (##sys#size s)) 
+			   (s s))
+		  (let* ((count (fxmin +output-chunk-size+ len))
+			 (n (##net#send fd s count 0)) )
+		    (cond ((eq? -1 n)
+			   (if (eq? errno _ewouldblock)
+			       (begin
+				 (##sys#thread-block-for-i/o! ##sys#current-thread fd #f)
+				 (yield) 
+				 (loop len s) )
+			       (begin
+				 (##sys#update-errno)
+				 (##sys#signal-hook 
+				  #:network-error (##sys#string-append "can not write to socket - " strerror) fd) ) ) )
+			  ((fx< n len)
+			   (loop (fx- len n) (##sys#substring s n (##sys#size s))) ) ) ) ) ) )
+	     (out
 	      (make-output-port
-	       (lambda (s) 
-		 (let ((len0 (##sys#size s)))
-		   (when (fx> len0 0)
-		     (let loop ((len len0) 
-				(s s) )
-		       (let* ((count (fxmin +output-chunk-size+ len))
-			      (n (##net#send fd s count 0)) )
-			 (cond [(eq? -1 n)
-				(if (eq? errno _ewouldblock)
-				    (begin
-				      (##sys#thread-block-for-i/o! ##sys#current-thread fd #f)
-				      (yield) 
-				      (loop len s) )
-				    (begin
-				      (##sys#update-errno)
-				      (##sys#signal-hook 
-				       #:network-error (##sys#string-append "can not write to socket - " strerror) fd) ) ) ]
-			       [(fx< n len)
-				(loop (fx- len n) (##sys#substring s n (##sys#size s))) ] ) ) ) ) ) )
+	       (if outbuf
+		   (lambda (s)
+		     (set! outbuf (##sys#string-append outbuf s))
+		     (when (fx>= (##sys#size outbuf) outbufsize)
+		       (output outbuf)
+		       (set! outbuf "") ) )
+		   (lambda (s) 
+		     (when (fx> (##sys#size s) 0)
+		       (output s)) ) )
 	       (lambda ()
 		 (unless oclosed
 		   (set! oclosed #t)
+		   (when (and outbuf (fx> (##sys#size outbuf) 0))
+		     (output outbuf)
+		     (set! outbuf "") )
 		   (unless (##sys#slot data 2) (##net#shutdown fd _sd_send))
 		   (when (and iclosed (eq? -1 (##net#close fd)))
 		     (##sys#update-errno)
 		     (##sys#signal-hook
-		      #:network-error (##sys#string-append "can not close socket output port - " strerror) fd) ) ) ) ) ] )
+		      #:network-error (##sys#string-append "can not close socket output port - " strerror) fd) ) ) )
+	       (and outbuf
+		    (lambda ()
+		      (when (fx> (##sys#size outbuf) 0)
+			(output outbuf)
+			(set! outbuf "") ) ) ) ) ) )
 	(##sys#setslot in 3 "(tcp)")
 	(##sys#setslot out 3 "(tcp)")
 	(##sys#setslot in 7 'socket)
@@ -411,10 +437,10 @@ EOF
 
 (define (tcp-accept tcpl)
   (##sys#check-structure tcpl 'tcp-listener)
-  (let ([fd (##sys#slot tcpl 1)])
+  (let ((fd (##sys#slot tcpl 1)))
     (let loop ()
       (if (eq? 1 (##net#select fd))
-	  (let ([fd (##net#accept fd #f #f)])
+	  (let ((fd (##net#accept fd #f #f)))
 	    (when (eq? -1 fd)
 	      (##sys#update-errno)
 	      (##sys#signal-hook 
@@ -428,7 +454,7 @@ EOF
 
 (define (tcp-accept-ready? tcpl)
   (##sys#check-structure tcpl 'tcp-listener 'tcp-accept-ready?)
-  (let ([f (##net#select (##sys#slot tcpl 1))])
+  (let ((f (##net#select (##sys#slot tcpl 1))))
     (when (eq? -1 f)
       (##sys#update-errno)
       (##sys#signal-hook 
@@ -447,14 +473,14 @@ EOF
 (define general-strerror (foreign-lambda c-string "strerror" int))
 
 (define (tcp-connect host . more)
-  (let ([port (:optional more #f)])
+  (let ((port (:optional more #f)))
     (##sys#check-string host)
     (unless port
       (set!-values (host port) (##net#parse-host host "tcp"))
       (unless port (##sys#signal-hook #:network-error 'tcp-connect "no port specified" host)) )
     (##sys#check-exact port)
-    (let ([addr (make-string _sockaddr_in_size)]
-	  [s (##net#socket _af_inet _sock_stream 0)] )
+    (let ((addr (make-string _sockaddr_in_size))
+	  (s (##net#socket _af_inet _sock_stream 0)) )
       (define (fail)
 	(##net#close s)
 	(##sys#update-errno)
@@ -472,7 +498,7 @@ EOF
       (when (eq? -1 (##net#connect s addr _sockaddr_in_size))
 	(if (eq? errno _einprogress)
 	    (let loop ()
-	      (let ([f (##net#select-write s)])
+	      (let ((f (##net#select-write s)))
 		(when (eq? f -1) (fail))
 		(unless (eq? f 1)
 		  ;(##sys#thread-block-for-i/o! ##sys#current-thread s #t)
@@ -491,13 +517,13 @@ EOF
 
 (define (##sys#tcp-port-data p)
   (##sys#check-port p)
-  (let ([d (##sys#port-data p)])
+  (let ((d (##sys#port-data p)))
     (if d
 	(##sys#slot d 0)
 	(##sys#signal-hook #:type-error "bad argument type - not a TCP port - " p) ) ) )
 
 (define (tcp-addresses p)
-  (let ([fd (##sys#tcp-port->fileno p)])
+  (let ((fd (##sys#tcp-port->fileno p)))
     (values 
      (or (##net#getsockname fd)
 	 (##sys#signal-hook #:network-error 'tcp-addresses (##sys#string-append "can not compute local address - " strerror) p) )
@@ -505,7 +531,7 @@ EOF
 	 (##sys#signal-hook #:network-error 'tcp-addresses (##sys#string-append "can not compute remote address - " strerror) p) ) ) ) )
 
 (define (tcp-port-numbers p)
- (let ([fd (##sys#tcp-port->fileno p)])
+ (let ((fd (##sys#tcp-port->fileno p)))
    (values
     (or (##net#getsockport fd)
         (##sys#signal-hook #:network-error 'tcp-port-numbers (##sys#string-append "can not compute local port - " strerror) p) )
@@ -514,8 +540,8 @@ EOF
 
 (define (tcp-listener-port tcpl)
   (##sys#check-structure tcpl 'tcp-listener 'tcp-listener-port)
-  (let* ([fd (##sys#slot tcpl 1)]
-	 [port (##net#getsockport fd)] )
+  (let* ((fd (##sys#slot tcpl 1))
+	 (port (##net#getsockport fd)) )
     (when (eq? -1 port)
       (##sys#signal-hook
        #:network-error 'tcp-listener-port (##sys#string-append "can not obtain listener port - " strerror) 
